/*
 *  Copyright (C) 2009  Giorgio Wicklein <g.wicklein@giowisys.com> 
 *  
 *  This file is part of Kontakter.
 *
 *  Kontakter is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Kontakter is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Kontakter.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QtGui>

#include "addresswidget.h"
#include "mainwidget.h"
#include "contactlistwidget.h"
#include "mainwindow.h"
#include "passwidget.h"
#include "settingswidget.h"
#include "definitions.h"

/* _________________________________PUBLIC__________________________________ */

//// constructor
AddressWidget::AddressWidget(QWidget *parent) : QWidget(parent)
{
//// create GUI
    createControls();
    setLayout(mainLayout);
    createCustomControls();

//// height is fixed but width is editable
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);

//// set the form non editable, so only display the info
    setEditable(false);

//// prepare the database
    bool prepareDB = prepareContactsDB();

//// load contacts from database to contactListWidget
    if (prepareDB)
        loadContactsDB();
}

//// creates the database and loads its configuration
bool AddressWidget::prepareContactsDB()
{
    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup("database");

//// if there are no databases
    if (!(settings.value("dbCount").toInt()))
        return false;

    db = new QSqlDatabase(QSqlDatabase::addDatabase("QSQLITE"));

//// update mainWindow's current Database string
    *mainWindow->currentDatabase = settings.value("currentDatabase").toString();

    QString dbName = QDir::homePath() + "/.Kontakter/databases/"
                     + (settings.value("currentDatabase").toString())
                     + ".sqlite";

    settings.endGroup();

    if(!(QFileInfo(dbName)).exists()) {
       QMessageBox::critical(this, tr("Database Error"),
                              tr("Could not find the database!"));
       return false;
    }

    db->setDatabaseName(dbName);


    if (!db->open()) {
        QMessageBox::critical(this, tr("Database Error"),
                              tr("Could not open the database!"));
        return false;
    }
    return true;
}

//// loads contacts from database to contactListWidget
bool AddressWidget::loadContactsDB()
{
    mw->contactListWidget->listWidget->clear();

    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup("database");

//// if there are no databases
    if (!(settings.value("dbCount").toInt()))
        return false;

    settings.endGroup();

    settings.beginGroup(*mainWindow->currentDatabase);
    if (settings.value("usePass").toBool()) {
        if (!(*validPass)) {
            checkPass();
            return false;
        }
    }
    settings.endGroup();

    QSqlQuery query(*db);

    query.exec("SELECT name, firstName FROM contacts ORDER BY name, firstName");
    while (query.next()) {
        mw->contactListWidget->addContact(QString(
                                  query.value(0).toString() +
                                  " " +
                                  query.value(1).toString()
                                  ));
    }
    return true;
}

void AddressWidget::print(int mode, bool pdf)
{
    /* MODES:
     * 1 -> print selected contact
     * 2 -> print all contacts
     */

    QPrinter printer;

    if (pdf) {
        QString outFile =
                QFileDialog::getSaveFileName(this, tr("Save PDF file"),
                                             QDir::homePath(),
                                             tr("PDF files (*.pdf)"));
        if (outFile.isEmpty())
            return;

        if (!outFile.contains(".pdf"))
            outFile.append(".pdf");

            printer.setOutputFileName(outFile);

    } else {
        QPrintDialog printDialog(&printer, this);

//// if user cancels the dialog
        if (!printDialog.exec())
            return;
    }

    QTextDocument document;
    QString html;
    QSqlQuery query(*db);

//// if print selected contact
    if (mode == 1) {
        QListWidgetItem *current = mw->contactListWidget->listWidget->currentItem();
        QString fullName = current->text();
        QStringList parts = fullName.split(" ", QString::SkipEmptyParts);
        QString name;
        QString firstName;

        QStringList::iterator iter = parts.begin();

        name = *iter;
        ++iter;
        if (!(parts.size() == 1)) {
            firstName = *iter;
            ++iter;
        } else {
            firstName = QString(""); // handle name==NULL
        }

        if (parts.length() > 2) {
            while (iter != parts.end()) {
                firstName.append(" " + (*iter));
                ++iter;
            }
        }

        query.prepare("SELECT * FROM contacts WHERE name=:name AND firstName=:firstName");
        query.bindValue(":name", name);
        query.bindValue(":firstName", firstName);
        query.exec();

    } else if (mode == 2) {
        query.exec("SELECT * FROM contacts ORDER BY name, firstName");
    }



    while (query.next()) {

        html += "<table width=\"100%\" border=1 cellspacing=0 cellpadding=\"5\">\n"
            "<tr><td><font size=\"18px\">"
            "<b>" + query.value(0).toString() + " "
            + query.value(1).toString() + "</b></font>\n"
            "<tr><td><font size=\"15px\">";

        bool work = false;
        bool home = false;
        bool mobile = false;
        bool addr = false;
        bool zip = false;
        bool town = false;
        bool email = false;
        bool web = false;
        bool notes = false;

        if (!(query.value(2).toString().isEmpty())) {
            html += query.value(2).toString();
            work = true;
        }
        if (!(query.value(3).toString().isEmpty())) {
            if (work)
                html += " | ";
            html += query.value(3).toString();
            home = true;
        }
        if (!(query.value(4).toString().isEmpty())) {
            if (work || home)
                html += " | ";
            html += query.value(4).toString();
            mobile = true;
        }
        if (mobile || work || home)
            html += "<br>";
        if (!(query.value(5).toString().isEmpty())) {
            html += query.value(5).toString() + " ";
            addr = true;
        }

        if (!(query.value(6).toString().isEmpty())) {
            if (addr)
                html += " | ";
            html += query.value(6).toString() + " ";
            zip = true;
        }
        if (!(query.value(7).toString().isEmpty())) {
            html += query.value(7).toString();
            town = true;
        }
        if (town || zip || addr)
            html += "<br>";
        if (!(query.value(8).toString().isEmpty())) {
            html += query.value(8).toString() + " ";
            email = true;
        }
        if (!(query.value(9).toString().isEmpty())) {
            html += query.value(9).toString();
            web = true;
        }
        if (web || email)
            html += "<br>";
        if (!(query.value(10).toString().isEmpty())) {
            html += query.value(10).toString();
            notes = true;
        }
        if (notes)
            html += "<br>";

        /* handle cutom fields */
        if (query.value(11).toInt()) {

            if (!(query.value(12).toString().isEmpty()))
                html += query.value(12).toString() + " "
                        + query.value(13).toString() + "<br>";

            if (!(query.value(14).toString().isEmpty()))
                html += query.value(14).toString() + " "
                        + query.value(15).toString() + "<br>";

            if (!(query.value(16).toString().isEmpty()))
                html += query.value(16).toString() + " "
                        + query.value(17).toString() + "<br>";

            if (!(query.value(18).toString().isEmpty()))
                html += query.value(18).toString() + " "
                        + query.value(19).toString() + "<br>";

            if (!(query.value(20).toString().isEmpty()))
                html += query.value(20).toString() + " "
                        + query.value(21).toString() + "<br>";

            if (!(query.value(22).toString().isEmpty()))
                html += query.value(22).toString() + " "
                        + query.value(23).toString() + "<br>";

        }

        html += "\n</font></table>\n<br>\n";
    }

    document.setHtml(html);
    document.print(&printer);
    if (pdf)
        mainWindow->statusBar()->showMessage(tr("PDF exported successfully!"));
    else
        mainWindow->statusBar()->showMessage(tr("Printing complete!"));
}

//// bool all:
//// true -> export all contacts
//// false -> export selected contact
void AddressWidget::exportvCard(bool all)
{
    QString outFileName =
                QFileDialog::getSaveFileName(this, tr("Save vCard"),
                                             QDir::homePath(),
                                             tr("vCard files (*.vcf)"));
    if (outFileName.isEmpty())
        return;

    if (!outFileName.contains(".vcf"))
        outFileName.append(".vcf");

    QFile outFile(outFileName);

    if (!outFile.open(QFile::WriteOnly | QFile::Text)) {
        QMessageBox::critical(this, tr("Error while saving"),
                              tr("An error occurred:\n"
                                 "cannot write data on selected file"));
        return;
    }

    QTextStream out(&outFile);
    QSqlQuery query(*db);

    if (!all) {

        QListWidgetItem *current = mw->contactListWidget->listWidget->currentItem();
        QString fullName = current->text();
        QStringList parts = fullName.split(" ", QString::SkipEmptyParts);
        QString name;
        QString firstName;

        QStringList::iterator iter = parts.begin();

        name = *iter;
        ++iter;
        if (!(parts.size() == 1)) {
            firstName = *iter;
            ++iter;
        } else {
            firstName = QString(""); // handle name==NULL
        }

        if (parts.length() > 2) {
            while (iter != parts.end()) {
                firstName.append(" " + (*iter));
                ++iter;
            }
        }

        query.prepare("SELECT * FROM contacts WHERE name=:name AND firstName=:firstName");
        query.bindValue(":name", name);
        query.bindValue(":firstName", firstName);
        query.exec();

    } else {

        query.exec("SELECT * FROM contacts ORDER BY name, firstName");

    }

    while (query.next()) {
        out << "BEGIN:VCARD\nVERSION:3.0\n"
            << "N:" << query.value(0).toString()
            << ";" << query.value(1).toString() << ";;;\n"
            << "FN:" << query.value(1).toString() << " "
            << query.value(0).toString() << "\n"
            << "EMAIL;type=INTERNET;type=WORK:"
            << query.value(8).toString() << "\n"
            << "TEL;type=WORK:" << query.value(2).toString() << "\n"
            << "TEL;type=CELL:" << query.value(4).toString() << "\n"
            << "TEL;type=HOME:" << query.value(3).toString() << "\n"
            << "ADR;type=WORK:;;" << query.value(5).toString() << ";"
            << query.value(7).toString() << ";;" << query.value(6).toString()
            << ";\n" << "URL;TYPE=HOME:" << query.value(9).toString() << "\n"
            << "NOTE:" << query.value(10).toString().replace(QString("\n"), QString(" ")) << "\n";

        /* Handle custom fields */
        if (query.value(11).toInt()) {
            if (!query.value(12).toString().isEmpty()) {
                out << "Label1:"
                    << query.value(12).toString().remove(":")
                    << "\nValue1:"
                    << query.value(13).toString()
                    << "\n";
            }
            if (!query.value(14).toString().isEmpty()) {
                out << "Label2:"
                    << query.value(14).toString().remove(":")
                    << "\nValue2:"
                    << query.value(15).toString()
                    << "\n";
            }
            if (!query.value(16).toString().isEmpty()) {
                out << "Label3:"
                    << query.value(16).toString().remove(":")
                    << "\nValue3:"
                    << query.value(17).toString()
                    << "\n";
            }
            if (!query.value(18).toString().isEmpty()) {
                out << "Label4:"
                    << query.value(18).toString().remove(":")
                    << "\nValue4:"
                    << query.value(19).toString()
                    << "\n";
            }
            if (!query.value(20).toString().isEmpty()) {
                out << "Label5:"
                    << query.value(20).toString().remove(":")
                    << "\nValue5:"
                    << query.value(21).toString()
                    << "\n";
            }
            if (!query.value(22).toString().isEmpty()) {
                out << "Label6:"
                    << query.value(22).toString().remove(":")
                    << "\nValue6:"
                    << query.value(23).toString().remove(":")
                    << "\n";
            }
        }
        out << "END:VCARD";
        if (all)
            out << "\n";
    }
    mainWindow->statusBar()->showMessage(tr("vCard exported successfully!"));
}

/* _________________________________________________________________________ */
/* ______________________________PUBLIC SLOTS_______________________________ */

void AddressWidget::addCustomEditField()
{
    if (*customCount == 5) {
        addButton->setEnabled(false);
    }

    lineLabelEditsVec->at(*currentCustomCount)->setVisible(true);
    lineLabelEditsVec->at(*currentCustomCount)->setText(tr("Label:"));
    lineEditsVec->at(*currentCustomCount)->setVisible(true);
    lineEditsVec->at(*currentCustomCount)->setText(tr("value"));
    deleteToolButtonsVec->at(*currentCustomCount)->setVisible(true);

    (*customCount)++;
    (*currentCustomCount)++;
}

//// prepare the form to create a new contact
void AddressWidget::addContact()
{
    if (mainWindow->accountType == "network") {

        if (!mainWindow->isRevUpToDate())
            return;

    }

    clearContactWidget();
    setEditable(true);
    nameLineEdit->setText(tr("Name"));
    firstNameLineEdit->setText(tr("First Name"));
}

void AddressWidget::editContactSlot()
{
    if (mainWindow->accountType == "network") {

        if (!mainWindow->isRevUpToDate())
            return;

    }

    *editContact = true;
    setEditable(true);
}

void AddressWidget::searchSlot(QString searchText)
{
    mw->contactListWidget->listWidget->clear();

    QSqlQuery query(*db);

    query.exec("SELECT * FROM contacts WHERE name LIKE '%"
                +searchText+"%' OR firstName LIKE '%"
                +searchText+"%' OR work LIKE '%"
                +searchText+"%' OR home LIKE '%"
                +searchText+"%' OR mobile LIKE '%"
                +searchText+"%' OR address LIKE '%"
                +searchText+"%' OR zip LIKE '%"
                +searchText+"%' OR town LIKE '%"
                +searchText+"%' OR email LIKE '%"
                +searchText+"%' OR web LIKE '%"
                +searchText+"%' OR notes LIKE '%"
                +searchText+"%' OR customLabel1 LIKE '%"
                +searchText+"%' OR customValue1 LIKE '%"
                +searchText+"%' OR customLabel2 LIKE '%"
                +searchText+"%' OR customValue2 LIKE '%"
                +searchText+"%' OR customLabel3 LIKE '%"
                +searchText+"%' OR customValue3 LIKE '%"
                +searchText+"%' OR customLabel4 LIKE '%"
                +searchText+"%' OR customValue4 LIKE '%"
                +searchText+"%' OR customLabel5 LIKE '%"
                +searchText+"%' OR customValue5 LIKE '%"
                +searchText+"%' OR customLabel6 LIKE '%"
                +searchText+"%' OR customValue6 LIKE '%"
                +searchText+"%'");

    while (query.next()) {
        mw->contactListWidget->addContact(QString(
                                  query.value(0).toString() +
                                  " " +
                                  query.value(1).toString()
                                  ));
    }
}

void AddressWidget::importvCard()
{
    QString inFileName = QFileDialog::getOpenFileName(this, tr("Import vCard file"),
                                             QDir::homePath(),
                                             tr("vCard files (*.vcf)"));
    if (inFileName.isEmpty())
        return;

    QFile inFile(inFileName);

    if (!inFile.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::critical(this, tr("Error while opening file"),
                              tr("An error occurred:\n"
                                 "cannot read data from selected file"));
        return;
    }

    QTextStream in(&inFile);
    QString line;
    clearContactWidget();

//// Parse vCard format
    while (!in.atEnd()) {
        line = in.readLine();
        if (line == QString("BEGIN:VCARD")) {
            line = in.readLine();
            if (line != QString("VERSION:3.0"))
                return;
            int customFields = 0;
            while (line != QString("END:VCARD")) {
                if (in.atEnd())
                    return;
                line = in.readLine();

                // switch fields
                QString field = line.split(":").at(0);

                if (field.contains("X-")) {
                    continue; // don't handle extra extended fields
                } else if (field == QString("N")) {
                    line.remove("N:");
                    QStringList name = line.split(";", QString::SkipEmptyParts);
                    QString lastName = name.at(0);
                    QString firstName;
                    int i = 1; // 1 because already have lastName
                    while (i < name.size())
                        firstName.append(name.at(i++));

                    nameLineEdit->setText(lastName);
                    firstNameLineEdit->setText(firstName);
                } else if (field.contains("EMAIL")) {
                    email1LineEdit->setText(line.split(":").at(1));;
                } else if (field.contains("TEL")) {
                    if (field.contains("WORK"))
                        workLineEdit->setText(line.split(":").at(1));
                    else if (field.contains("HOME"))
                        homeLineEdit->setText(line.split(":").at(1));
                    else if (field.contains("CELL"))
                        mobileLineEdit->setText(line.split(":").at(1));
                    else
                        homeLineEdit->setText(line.split(":").at(1));
                } else if (field.contains("ADR")) {
                    QString addrUnParsed = line.split(":").at(1);
                    QStringList addrParts = addrUnParsed.split(";", QString::KeepEmptyParts);
                    if ((addrParts.at(1)).isEmpty()) {
                        addressLineEdit->setText(addrParts.at(2));
                        townLineEdit->setText(addrParts.at(3));
                        zipLineEdit->setText(addrParts.at(5));
                    //// handle if ;addres here;;; like google mail
                    } else {
                        addressLineEdit->setText(QString(addrParts.at(1)).remove(";"));
                    }
                } else if (field.contains("URL")) {
                    webLineEdit->setText(line.split(":").at(1));
                } else if (field.contains("NOTE")) {
                    noteTextEdit->setPlainText(line.remove("NOTE:"));
                } else if (field.contains("Label1")) {
                    customLabelEdit1->setText(line.remove("Label1:"));
                    customFields++;
                } else if (field.contains("Value1")) {
                    customValueEdit1->setText(line.remove("Value1:"));
                } else if (field.contains("Label2")) {
                    customLabelEdit2->setText(line.remove("Label2:"));
                    customFields++;
                } else if (field.contains("Value2")) {
                    customValueEdit2->setText(line.remove("Value2:"));
                } else if (field.contains("Label3")) {
                    customLabelEdit3->setText(line.remove("Label3:"));
                    customFields++;
                } else if (field.contains("Value3")) {
                    customValueEdit3->setText(line.remove("Value3:"));
                } else if (field.contains("Label4")) {
                    customLabelEdit4->setText(line.remove("Label4:"));
                    customFields++;
                } else if (field.contains("Value4")) {
                    customValueEdit4->setText(line.remove("Value4:"));
                } else if (field.contains("Label5")) {
                    customLabelEdit5->setText(line.remove("Label5:"));
                    customFields++;
                } else if (field.contains("Value5")) {
                    customValueEdit5->setText(line.remove("Value5:"));
                } else if (field.contains("Label6")) {
                    customLabelEdit6->setText(line.remove("Label6:"));
                    customFields++;
                } else if (field.contains("Value6")) {
                    customValueEdit6->setText(line.remove("Value6:"));
                }
            }
            *customCount = customFields;
        }
        saveEditSlot();
    }
    mainWindow->statusBar()->showMessage(tr("vCard imported successfully!"));
}

/* _________________________________________________________________________ */
/* ______________________________PRIVATE SLOTS______________________________ */

//// activated when user
//// cancels the add/edit
void AddressWidget::cancelEditSlot()
{
//// cancel edit contact
    if (*editContact) {
        *editContact = false;
        setEditable(false);
        selectionChangedSlot();
        return ;
    }

//// cancel add contact
    int r = QMessageBox::question(this,
                                  tr("Cancel"),
                                  tr("Are you sure you want to cancel your changes?"),
                                  QMessageBox::No | QMessageBox::Yes);
    if (r == QMessageBox::No)
        return;

    clearContactWidget();
    mainWindow->updateActions();
}

void AddressWidget::saveEditSlot()
{
//// abort if name is empty
    if (QString(nameLineEdit->text()).isEmpty()) {
        QMessageBox::warning(this, tr("Error"),
                             tr("The contact's name cannot be empty!"));
        return;
    }

//// save new contact
    if (!(*editContact)) {
        if (!addContactToDB()) {
            return;
        }
        emit addContactListSignal(QString(nameLineEdit->text() + " " +
                                      firstNameLineEdit->text()));
        clearContactWidget();
    }
//// save edited contact
    if (*editContact) {
        saveEdittedContact();
        *editContact = false;
        QListWidgetItem *current = mw->contactListWidget->listWidget->currentItem();
        current->setText(QString(nameLineEdit->text() +
                                 " " + firstNameLineEdit->text()));
        selectionChangedSlot();
    }

    setEditable(false);
    mainWindow->updateActions();
}

void AddressWidget::deleteContactSlot()
{
    if (mainWindow->accountType == "network") {

        if (!mainWindow->isRevUpToDate())
            return;
    }

    QListWidgetItem *current = mw->contactListWidget->listWidget->currentItem();
    QString fullName = current->text();

    int r = QMessageBox::question(this, tr("Delete Contact"),
                                   tr("Are your sure you want to delete: ")
                                   + fullName,
                                   QMessageBox::No | QMessageBox::Yes);
    if ( r == QMessageBox::No)
        return;

    QStringList parts = fullName.split(" ", QString::SkipEmptyParts);
    QString name;
    QString firstName;
        
    /* handle if firstName is empty */
    if (firstNameLineEdit->text().isEmpty()) {
        name = parts.at(0);
        firstName = "";
    /* if not empty parse name and firstName */
    } else {
        QStringList::iterator iter = parts.begin();

        name = *iter;
        ++iter;
        firstName = *iter;
        ++iter;

        if (parts.length() > 2) {
            while (iter != parts.end()) {
                firstName.append(" " + (*iter));
                ++iter;
            }
        }
    }

    mw->contactListWidget->listWidget->removeItemWidget(current);
    if (mw->contactListWidget->listWidget->count() <= 1)
        mw->contactListWidget->listWidget->clear();
    else {
        delete current;
    }

    if (mainWindow->accountType == "network") {

        if (!mainWindow->connectToServer())
            return;

        QByteArray delContact;
        QDataStream outData(&delContact, QIODevice::WriteOnly);
        outData.setVersion(QDataStream::Qt_4_5);

        outData << name << firstName;

        QByteArray outBlock;
        QDataStream out(&outBlock, QIODevice::WriteOnly);
        out.setVersion(QDataStream::Qt_4_5);
        out << qint64(0);
        out << (*mainWindow->localServerPass) << quint16(DEL_REQUEST) << delContact;
        out.device()->seek(0);
        out << qint64(outBlock.size() - sizeof(qint64));

        mainWindow->statusBar()->showMessage(tr("Sending DEL request to server..."));

        mainWindow->tcpSocket->write(outBlock);

        if (!mainWindow->tcpSocket->waitForBytesWritten()) {
            mainWindow->statusBar()->showMessage(tr("Server refused DEL request!"));
            return;
        }

        if (!mainWindow->tcpSocket->waitForReadyRead())
            return;

        QDataStream in(mainWindow->tcpSocket);
        in.setVersion(QDataStream::Qt_4_5);

        /* Kontakter packet format */
        qint64 size;
        QString serverPass;
        quint16 reqType;
        QByteArray data;

        in >> size >> serverPass >> reqType >> data;

        if (reqType == DEL_REQUEST) {

            mainWindow->statusBar()->showMessage(tr("Contact deleted successfully!"));

        } else if (reqType == ERROR_REPORT) {

            QMessageBox::critical(this, tr("Error"), QString(data));
            mainWindow->statusBar()->showMessage(tr("Error!"));
            mainWindow->disconnectFromServer();
            return;

        } else {
            mainWindow->statusBar()->showMessage(tr("Request Type Error!"));
            mainWindow->disconnectFromServer();
            return;
        }

        mainWindow->incrementLocalRevision();

        mainWindow->disconnectFromServer();
    }

    QSqlQuery query(*db);

    query.prepare("DELETE FROM contacts WHERE name=:name AND firstName=:firstName");
    query.bindValue(":name", name);
    query.bindValue(":firstName", firstName);
    query.exec();
    
    clearContactWidget();
    selectionChangedSlot();

    emit updateActionsSignal();
}

//// handles deleteToolButton
//// to remove custom controls
void AddressWidget::handleRemoveCustom()
{
    (*customCount)--;
}

//// activated whenever user changes selection
//// in contactListWidget to update the form
void AddressWidget::selectionChangedSlot()
{
//// no contacts?
    if (mw->contactListWidget->listWidget->count() < 1)
        return;

    clearContactWidget();

//// extract name and firstName from contactList
    QListWidgetItem *current = mw->contactListWidget->listWidget->currentItem();
    QString fullName = current->text();
    QStringList parts = fullName.split(" ", QString::SkipEmptyParts);
    QString name;
    QString firstName;

    QStringList::iterator iter = parts.begin();

    name = *iter;
    ++iter;
    if (!(parts.size() == 1)) {
        firstName = *iter;
        ++iter;
    } else {
        firstName = QString(""); // handle name==NULL
    }

    if (parts.length() > 2) {
        while (iter != parts.end()) {
            firstName.append(" " + (*iter));
            ++iter;
        }
    }

    QSqlQuery query(*db);

    query.prepare("SELECT * FROM contacts WHERE name=:name AND firstName=:firstName");
    query.bindValue(":name", name);
    query.bindValue(":firstName", firstName);
    query.exec();
    query.next(); //needed to avoid error

    nameLineEdit->setText(query.value(0).toString());
    firstNameLineEdit->setText(query.value(1).toString());
    workLineEdit->setText(query.value(2).toString());
    homeLineEdit->setText(query.value(3).toString());
    mobileLineEdit->setText(query.value(4).toString());
    addressLineEdit->setText(query.value(5).toString());
    zipLineEdit->setText(query.value(6).toString());
    townLineEdit->setText(query.value(7).toString());
    email1LineEdit->setText(query.value(8).toString());
    webLineEdit->setText(query.value(9).toString());
    noteTextEdit->setPlainText(query.value(10).toString());
    *selectedContactId = query.value(24).toInt();
    *customCount = query.value(11).toInt();
    *currentCustomCount = 0;

//// handle custom content
    if (*customCount) {
        int query_custom_value = 12;
        while (*currentCustomCount < *customCount) {
            lineLabelEditsVec->at(*currentCustomCount)->setVisible(true);
            lineLabelEditsVec->at(*currentCustomCount)->setText(
                    query.value(query_custom_value).toString()
                    );
            ++query_custom_value;
            lineEditsVec->at(*currentCustomCount)->setVisible(true);
            lineEditsVec->at(*currentCustomCount)->setText(
                    query.value(query_custom_value).toString()
                    );
            (*currentCustomCount)++;
            ++query_custom_value;
        }
    }
}

void AddressWidget::mapButtonSlot()
{
    QDesktopServices::openUrl(
            QUrl("http://maps.google.com/maps?f=q&source=s_q&hl=en&geocode=&q="
                 + addressLineEdit->text() + " " + zipLineEdit->text() + " " + townLineEdit->text())
            );
}

void AddressWidget::emailButtonSlot()
{
    QDesktopServices::openUrl(QUrl("mailto:" + email1LineEdit->text()));
}

void AddressWidget::webButtonSlot()
{
    QDesktopServices::openUrl(QUrl("http://" + webLineEdit->text()));
}

void AddressWidget::setEditable(bool editable)
{
    editable = !editable;
    nameLineEdit->setReadOnly(editable);
    firstNameLineEdit->setReadOnly(editable);
    workLineEdit->setReadOnly(editable);
    homeLineEdit->setReadOnly(editable);
    mobileLineEdit->setReadOnly(editable);
    addressLineEdit->setReadOnly(editable);
    zipLineEdit->setReadOnly(editable);
    townLineEdit->setReadOnly(editable);
    email1LineEdit->setReadOnly(editable);
    webLineEdit->setReadOnly(editable);
    noteTextEdit->setReadOnly(editable);
    addButton->setEnabled(!editable);

    cancelButton->setVisible(!editable);
    saveButton->setVisible(!editable);
    mapButton->setVisible(editable);
    emailButton->setVisible(editable);
    webButton->setVisible(editable);

    customLabelEdit1->setReadOnly(editable);
    customLabelEdit2->setReadOnly(editable);
    customLabelEdit3->setReadOnly(editable);
    customLabelEdit4->setReadOnly(editable);
    customLabelEdit5->setReadOnly(editable);
    customLabelEdit6->setReadOnly(editable);

    customValueEdit1->setReadOnly(editable);
    customValueEdit2->setReadOnly(editable);
    customValueEdit3->setReadOnly(editable);
    customValueEdit4->setReadOnly(editable);
    customValueEdit5->setReadOnly(editable);
    customValueEdit6->setReadOnly(editable);


    if (*customCount) {
        *currentCustomCount = 0;

        while (*currentCustomCount < *customCount) {
            deleteToolButtonsVec->at(*currentCustomCount)->setVisible(!editable);
            (*currentCustomCount)++;
        }
    }

    emit editModeSignal(!editable);
}

void AddressWidget::checkPassSlot()
{
    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup(*mainWindow->currentDatabase);
    QString realPass = settings.value("pass").toByteArray().toHex();
    QByteArray inputHash = QCryptographicHash::hash(
                QByteArray(passWidget->passLineEdit->text().toStdString().c_str()),
                QCryptographicHash::Md5);
    QString inputPass = inputHash.toHex();
    settings.endGroup();

    if (inputPass == realPass) {
        *validPass = true;
        delete passWidget;
        loadContactsDB();
        mainWindow->settingsWidget->loadSettings();
    }
    else {
        *validPass = false;
        QMessageBox::warning(this, tr("Password request"),
                             tr("Wrong password!"));
    }

}

void AddressWidget::passCancelSlot()
{
    delete passWidget;
    mainWindow->disableAll(true);
    mainWindow->disableSyncAction();
}

/* _________________________________________________________________________ */
/* _________________________________PRIVATE_________________________________ */

void AddressWidget::createControls()
{
    editContact = new bool;
    validPass = new bool;
    *validPass = false;
    *editContact = false;
    selectedContactId = new quint64;
    contactLabel = new QLabel(this);
    contactLabel->setPixmap(QPixmap(":/icons/contact.png"));

    nameLineEdit = new QLineEdit(this);

//// exclude spaces
    QRegExp rx("^\\S+$");
    QValidator *validator = new QRegExpValidator(rx, this);
    nameLineEdit->setValidator(validator);

    firstNameLineEdit = new QLineEdit(this);

    workLabel = new QLabel(tr("Work: "), this);
    workLineEdit = new QLineEdit(this);
    homeLabel = new QLabel(tr("Home: "), this);
    homeLineEdit = new QLineEdit(this);
    mobileLabel = new QLabel(tr("Mobile: "), this);
    mobileLineEdit = new QLineEdit(this);
    addressLabel = new QLabel(tr("Address: "), this);
    addressLineEdit = new QLineEdit(this);
    zipLabel = new QLabel(tr("ZIP: "), this);
    zipLineEdit = new QLineEdit(this);
    townLabel = new QLabel(tr("Town: "), this);
    townLineEdit = new QLineEdit(this);
    email1Label = new QLabel(tr("e-mail: "), this);
    email1LineEdit = new QLineEdit(this);
    webLabel = new QLabel(tr("Web: "), this);
    webLineEdit = new QLineEdit(this);

    noteLabel = new QLabel(tr("Notes: "), this);
    noteTextEdit = new QPlainTextEdit(this);
    noteTextEdit->setReadOnly(true);

    addButton = new QPushButton(tr("Add custom field"), this);
    addButton->setEnabled(false);
    connect(addButton, SIGNAL(clicked()), this, SLOT(addCustomEditField()));

    customLayout = new QVBoxLayout;

    cancelButton = new QToolButton(this);
    cancelButton->setIcon(QIcon(":/icons/cancel.png"));
    cancelButton->setIconSize(QSize(16, 16));
    cancelButton->setStatusTip(tr("Cancel current changes"));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelEditSlot()));

    saveButton = new QToolButton(this);
    saveButton->setIcon(QIcon(":/icons/save.png"));
    saveButton->setIconSize(QSize(16, 16));
    saveButton->setStatusTip(tr("Save current changes"));
    saveButton->setShortcut(QKeySequence::Save);
    connect(saveButton, SIGNAL(clicked()), this, SLOT(saveEditSlot()));

    cancelButton->setVisible(false);
    saveButton->setVisible(false);

    editButtonsLayout = new QHBoxLayout;
    editButtonsLayout->addWidget(cancelButton);
    editButtonsLayout->addWidget(saveButton);

    mapButton = new QToolButton(this);
    mapButton->setIcon(QIcon(":/icons/map.png"));
    mapButton->setIconSize(QSize(16, 16));
    mapButton->setStatusTip(tr("Show address on Google Maps"));
    connect(mapButton, SIGNAL(clicked()), this, SLOT(mapButtonSlot()));

    emailButton = new QToolButton(this);
    emailButton->setIcon(QIcon(":/icons/mail.png"));
    emailButton->setIconSize(QSize(16, 16));
    emailButton->setStatusTip(tr("Send email"));
    connect(emailButton, SIGNAL(clicked()), this, SLOT(emailButtonSlot()));

    webButton = new QToolButton(this);
    webButton->setIcon(QIcon(":/icons/web.png"));
    webButton->setIconSize(QSize(16, 16));
    webButton->setStatusTip(tr("Show website"));
    connect(webButton, SIGNAL(clicked()), this, SLOT(webButtonSlot()));

    mainLayout = new QGridLayout;
    mainLayout->addWidget(contactLabel, 0, 0);
    mainLayout->addWidget(nameLineEdit, 0, 1, 1, 2, 0);
    mainLayout->addWidget(firstNameLineEdit, 0, 3);
    mainLayout->addWidget(workLabel, 1, 0);
    mainLayout->addWidget(workLineEdit, 1, 1, 1, 2, 0);
    mainLayout->addWidget(homeLabel, 2, 0);
    mainLayout->addWidget(homeLineEdit, 2, 1, 1, 2, 0);
    mainLayout->addWidget(mobileLabel, 3, 0);
    mainLayout->addWidget(mobileLineEdit, 3, 1, 1, 2, 0);
    mainLayout->addWidget(addressLabel, 4, 0);
    mainLayout->addWidget(addressLineEdit, 4, 1, 1, 2, 0);
    mainLayout->addWidget(zipLabel, 5, 0);
    mainLayout->addWidget(zipLineEdit, 5, 1, 1, 2, 0);
    mainLayout->addWidget(townLabel, 6, 0);
    mainLayout->addWidget(townLineEdit, 6, 1, 1, 1, 0);
    mainLayout->addWidget(mapButton, 6, 2);
    mainLayout->addWidget(email1Label, 7, 0);
    mainLayout->addWidget(email1LineEdit, 7, 1, 1, 1, 0);
    mainLayout->addWidget(emailButton, 7, 2);
    mainLayout->addWidget(webLabel, 8, 0);
    mainLayout->addWidget(webLineEdit, 8, 1, 1, 1, 0);
    mainLayout->addWidget(webButton, 8, 2);
    mainLayout->addWidget(noteLabel, 1, 3);
    mainLayout->addWidget(noteTextEdit, 2, 3, 6, 1, 0);
    mainLayout->addWidget(addButton, 8, 3);
    mainLayout->addLayout(customLayout, 9, 0, 1, 4, 0);
    mainLayout->addLayout(editButtonsLayout, 10, 0, 1, 1, 0);

    mw = qobject_cast<MainWidget*>(parentWidget());
    mainWindow = qobject_cast<MainWindow*>(mw->parentWidget());

    connect(mw->contactListWidget->listWidget, SIGNAL(itemSelectionChanged()),
            this, SLOT(selectionChangedSlot()));
}

void AddressWidget::createCustomControls()
{
    customCount = new int;
    currentCustomCount = new int;
    *customCount = 0;
    *currentCustomCount = 0;

    deleteToolButtonsVec = new QVector<QToolButton*>;
    lineLabelEditsVec = new QVector<QLineEdit*>;
    lineEditsVec = new QVector<QLineEdit*>;

    customLabelEdit1 = new QLineEdit(this);
    customLabelEdit2 = new QLineEdit(this);
    customLabelEdit3 = new QLineEdit(this);
    customLabelEdit4 = new QLineEdit(this);
    customLabelEdit5 = new QLineEdit(this);
    customLabelEdit6 = new QLineEdit(this);
    customValueEdit1 = new QLineEdit(this);
    customValueEdit2 = new QLineEdit(this);
    customValueEdit3 = new QLineEdit(this);
    customValueEdit4 = new QLineEdit(this);
    customValueEdit5 = new QLineEdit(this);
    customValueEdit6 = new QLineEdit(this);
    deleteToolButton1 = new QToolButton(this);
    deleteToolButton2 = new QToolButton(this);
    deleteToolButton3 = new QToolButton(this);
    deleteToolButton4 = new QToolButton(this);
    deleteToolButton5 = new QToolButton(this);
    deleteToolButton6 = new QToolButton(this);

    deleteToolButtonsVec->append(deleteToolButton1);
    deleteToolButtonsVec->append(deleteToolButton2);
    deleteToolButtonsVec->append(deleteToolButton3);
    deleteToolButtonsVec->append(deleteToolButton4);
    deleteToolButtonsVec->append(deleteToolButton5);
    deleteToolButtonsVec->append(deleteToolButton6);
    lineLabelEditsVec->append(customLabelEdit1);
    lineLabelEditsVec->append(customLabelEdit2);
    lineLabelEditsVec->append(customLabelEdit3);
    lineLabelEditsVec->append(customLabelEdit4);
    lineLabelEditsVec->append(customLabelEdit5);
    lineLabelEditsVec->append(customLabelEdit6);
    lineEditsVec->append(customValueEdit1);
    lineEditsVec->append(customValueEdit2);
    lineEditsVec->append(customValueEdit3);
    lineEditsVec->append(customValueEdit4);
    lineEditsVec->append(customValueEdit5);
    lineEditsVec->append(customValueEdit6);

    deleteToolButton1->setIcon(QIcon(":/icons/delete_contact.png"));
    deleteToolButton1->setIconSize(QSize(16, 16));
    deleteToolButton1->setVisible(false);
    deleteToolButton2->setIcon(QIcon(":/icons/delete_contact.png"));
    deleteToolButton2->setIconSize(QSize(16, 16));
    deleteToolButton2->setVisible(false);
    deleteToolButton3->setIcon(QIcon(":/icons/delete_contact.png"));
    deleteToolButton3->setIconSize(QSize(16, 16));
    deleteToolButton3->setVisible(false);
    deleteToolButton4->setIcon(QIcon(":/icons/delete_contact.png"));
    deleteToolButton4->setIconSize(QSize(16, 16));
    deleteToolButton4->setVisible(false);
    deleteToolButton5->setIcon(QIcon(":/icons/delete_contact.png"));
    deleteToolButton5->setIconSize(QSize(16, 16));
    deleteToolButton5->setVisible(false);
    deleteToolButton6->setIcon(QIcon(":/icons/delete_contact.png"));
    deleteToolButton6->setIconSize(QSize(16, 16));
    deleteToolButton6->setVisible(false);

    customLabelEdit1->setVisible(false);
    customLabelEdit2->setVisible(false);
    customLabelEdit3->setVisible(false);
    customLabelEdit4->setVisible(false);
    customLabelEdit5->setVisible(false);
    customLabelEdit6->setVisible(false);

    customValueEdit1->setVisible(false);
    customValueEdit2->setVisible(false);
    customValueEdit3->setVisible(false);
    customValueEdit4->setVisible(false);
    customValueEdit5->setVisible(false);
    customValueEdit6->setVisible(false);

    connect(deleteToolButton1, SIGNAL(clicked()),
            customLabelEdit1, SLOT(hide()));
    connect(deleteToolButton1, SIGNAL(clicked()),
            customValueEdit1, SLOT(hide()));
    connect(deleteToolButton1, SIGNAL(clicked()),
            deleteToolButton1, SLOT(hide()));
    connect(deleteToolButton1, SIGNAL(clicked()),
            customLabelEdit1, SLOT(clear()));
    connect(deleteToolButton1, SIGNAL(clicked()),
            customValueEdit1, SLOT(clear()));
    connect(deleteToolButton1, SIGNAL(clicked()),
            this, SLOT(handleRemoveCustom()));

    connect(deleteToolButton2, SIGNAL(clicked()),
            customLabelEdit2, SLOT(hide()));
    connect(deleteToolButton2, SIGNAL(clicked()),
            customValueEdit2, SLOT(hide()));
    connect(deleteToolButton2, SIGNAL(clicked()),
            deleteToolButton2, SLOT(hide()));
    connect(deleteToolButton2, SIGNAL(clicked()),
            customLabelEdit2, SLOT(clear()));
    connect(deleteToolButton2, SIGNAL(clicked()),
            customValueEdit2, SLOT(clear()));
    connect(deleteToolButton2, SIGNAL(clicked()),
            this, SLOT(handleRemoveCustom()));

    connect(deleteToolButton3, SIGNAL(clicked()),
            customLabelEdit3, SLOT(hide()));
    connect(deleteToolButton3, SIGNAL(clicked()),
            customValueEdit3, SLOT(hide()));
    connect(deleteToolButton3, SIGNAL(clicked()),
            deleteToolButton3, SLOT(hide()));
    connect(deleteToolButton3, SIGNAL(clicked()),
            customLabelEdit3, SLOT(clear()));
    connect(deleteToolButton3, SIGNAL(clicked()),
            customValueEdit3, SLOT(clear()));
    connect(deleteToolButton3, SIGNAL(clicked()),
            this, SLOT(handleRemoveCustom()));

    connect(deleteToolButton4, SIGNAL(clicked()),
            customLabelEdit4, SLOT(hide()));
    connect(deleteToolButton4, SIGNAL(clicked()),
            customValueEdit4, SLOT(hide()));
    connect(deleteToolButton4, SIGNAL(clicked()),
            deleteToolButton4, SLOT(hide()));
    connect(deleteToolButton4, SIGNAL(clicked()),
            customLabelEdit4, SLOT(clear()));
    connect(deleteToolButton4, SIGNAL(clicked()),
            customValueEdit4, SLOT(clear()));
    connect(deleteToolButton4, SIGNAL(clicked()),
            this, SLOT(handleRemoveCustom()));

    connect(deleteToolButton5, SIGNAL(clicked()),
            customLabelEdit5, SLOT(hide()));
    connect(deleteToolButton5, SIGNAL(clicked()),
            customValueEdit5, SLOT(hide()));
    connect(deleteToolButton5, SIGNAL(clicked()),
            deleteToolButton5, SLOT(hide()));
    connect(deleteToolButton5, SIGNAL(clicked()),
            customLabelEdit5, SLOT(clear()));
    connect(deleteToolButton5, SIGNAL(clicked()),
            customValueEdit5, SLOT(clear()));
    connect(deleteToolButton5, SIGNAL(clicked()),
            this, SLOT(handleRemoveCustom()));

    connect(deleteToolButton6, SIGNAL(clicked()),
            customLabelEdit6, SLOT(hide()));
    connect(deleteToolButton6, SIGNAL(clicked()),
            customValueEdit6, SLOT(hide()));
    connect(deleteToolButton6, SIGNAL(clicked()),
            deleteToolButton6, SLOT(hide()));
    connect(deleteToolButton6, SIGNAL(clicked()),
            customLabelEdit6, SLOT(clear()));
    connect(deleteToolButton6, SIGNAL(clicked()),
            customValueEdit6, SLOT(clear()));
    connect(deleteToolButton6, SIGNAL(clicked()),
            this, SLOT(handleRemoveCustom()));

    customLayout1 = new QHBoxLayout;
    customLayout2 = new QHBoxLayout;
    customLayout3 = new QHBoxLayout;
    customLayout4 = new QHBoxLayout;
    customLayout5 = new QHBoxLayout;
    customLayout6 = new QHBoxLayout;

    customLayout1->addWidget(customLabelEdit1);
    customLayout1->addWidget(customValueEdit1);
    customLayout1->addWidget(deleteToolButton1);
    customLayout->addLayout(customLayout1);

    customLayout2->addWidget(customLabelEdit2);
    customLayout2->addWidget(customValueEdit2);
    customLayout2->addWidget(deleteToolButton2);
    customLayout->addLayout(customLayout2);

    customLayout3->addWidget(customLabelEdit3);
    customLayout3->addWidget(customValueEdit3);
    customLayout3->addWidget(deleteToolButton3);
    customLayout->addLayout(customLayout3);

    customLayout4->addWidget(customLabelEdit4);
    customLayout4->addWidget(customValueEdit4);
    customLayout4->addWidget(deleteToolButton4);
    customLayout->addLayout(customLayout4);

    customLayout5->addWidget(customLabelEdit5);
    customLayout5->addWidget(customValueEdit5);
    customLayout5->addWidget(deleteToolButton5);
    customLayout->addLayout(customLayout5);

    customLayout6->addWidget(customLabelEdit6);
    customLayout6->addWidget(customValueEdit6);
    customLayout6->addWidget(deleteToolButton6);
    customLayout->addLayout(customLayout6);
}



void AddressWidget::clearContactWidget()
{
    nameLineEdit->clear();
    firstNameLineEdit->clear();
    workLineEdit->clear();
    homeLineEdit->clear();
    mobileLineEdit->clear();
    addressLineEdit->clear();
    zipLineEdit->clear();
    townLineEdit->clear();
    email1LineEdit->clear();
    webLineEdit->clear();
    noteTextEdit->clear();

    if (*customCount) {
        int i = 0;
        while (i < *customCount) {
            lineLabelEditsVec->at(i)->setVisible(false);
            lineEditsVec->at(i)->setVisible(false);
            deleteToolButtonsVec->at(i)->setVisible(false);
            ++i;
        }
        *customCount = 0;
        *currentCustomCount = 0;
    }

    setEditable(false);
}

void AddressWidget::saveEdittedContact()
{

    if (mainWindow->accountType == "network") {

        if (!mainWindow->isRevUpToDate())
            return;

        if (!mainWindow->connectToServer())
            return;

        mainWindow->statusBar()->showMessage(tr("Sending editted contact to server..."));

        QByteArray editContact;
        QDataStream outData(&editContact, QIODevice::WriteOnly);
        outData.setVersion(QDataStream::Qt_4_5);

        outData << nameLineEdit->text() << firstNameLineEdit->text() << workLineEdit->text()
                << homeLineEdit->text() << mobileLineEdit->text() << addressLineEdit->text()
                << zipLineEdit->text() << townLineEdit->text() << email1LineEdit->text()
                << webLineEdit->text() << noteTextEdit->toPlainText() << (*customCount)
                << customLabelEdit1->text() << customValueEdit1->text() << customLabelEdit2->text()
                << customValueEdit2->text() << customLabelEdit3->text() << customValueEdit3->text()
                << customLabelEdit4->text() << customValueEdit4->text() << customLabelEdit5->text()
                << customValueEdit5->text() << customLabelEdit6->text() << customValueEdit6->text()
                << (*selectedContactId);

        QByteArray outBlock;
        QDataStream out(&outBlock, QIODevice::WriteOnly);
        out.setVersion(QDataStream::Qt_4_5);
        out << qint64(0);
        out << (*mainWindow->localServerPass) << quint16(EDIT_REQUEST) << editContact;
        out.device()->seek(0);
        out << qint64(outBlock.size() - sizeof(qint64));

        mainWindow->statusBar()->showMessage(tr("Sending EDIT request to server..."));

        mainWindow->tcpSocket->write(outBlock);

        if (!mainWindow->tcpSocket->waitForBytesWritten()) {
            mainWindow->statusBar()->showMessage(tr("Server refused EDIT request!"));
            return;
        }

        if (!mainWindow->tcpSocket->waitForReadyRead())
            return;

        QDataStream in(mainWindow->tcpSocket);
        in.setVersion(QDataStream::Qt_4_5);

        /* Kontakter packet format */
        qint64 size;
        QString serverPass;
        quint16 reqType;
        QByteArray data;

        in >> size >> serverPass >> reqType >> data;

        if (reqType == EDIT_REQUEST) {

            mainWindow->statusBar()->showMessage(tr("Contact editted successfully!"));

        } else if (reqType == ERROR_REPORT) {

            QMessageBox::critical(this, tr("Error"), QString(data));
            mainWindow->statusBar()->showMessage(tr("Error!"));
            mainWindow->disconnectFromServer();
            return;

        } else {
            mainWindow->statusBar()->showMessage(tr("Request Type Error!"));
            mainWindow->disconnectFromServer();
            return;
        }

        mainWindow->incrementLocalRevision();

        mainWindow->disconnectFromServer();
    }

    QSqlQuery query(*db);
    query.prepare(
            "UPDATE contacts SET "
            "name=:name, firstName=:firstName, work=:work, home=:home, mobile=:mobile, address=:address, zip=:zip, town=:town, email=:email, web=:web, notes=:notes, customCount=:customCount, customLabel1=:customLabel1, customValue1=:customValue1, customLabel2=:customLabel2, customValue2=:customValue2, customLabel3=:customLabel3, customValue3=:customValue3, customLabel4=:customLabel4, customValue4=:customValue4, customLabel5=:customLabel5, customValue5=:customValue5, customLabel6=:customLabel6, customValue6=:customValue6, date=CURRENT_TIMESTAMP"
            " WHERE id=:id"
            );
    query.bindValue(":name", nameLineEdit->text());
    query.bindValue(":firstName", firstNameLineEdit->text());
    query.bindValue(":work", workLineEdit->text());
    query.bindValue(":home", homeLineEdit->text());
    query.bindValue(":mobile", mobileLineEdit->text());
    query.bindValue(":address", addressLineEdit->text());
    query.bindValue(":zip", zipLineEdit->text());
    query.bindValue(":town", townLineEdit->text());
    query.bindValue(":email", email1LineEdit->text());
    query.bindValue(":web", webLineEdit->text());
    query.bindValue(":notes", noteTextEdit->toPlainText());
    query.bindValue(":customCount", *customCount);
    query.bindValue(":customLabel1", customLabelEdit1->text());
    query.bindValue(":customValue1", customValueEdit1->text());
    query.bindValue(":customLabel2", customLabelEdit2->text());
    query.bindValue(":customValue2", customValueEdit2->text());
    query.bindValue(":customLabel3", customLabelEdit3->text());
    query.bindValue(":customValue3", customValueEdit3->text());
    query.bindValue(":customLabel4", customLabelEdit4->text());
    query.bindValue(":customValue4", customValueEdit4->text());
    query.bindValue(":customLabel5", customLabelEdit5->text());
    query.bindValue(":customValue5", customValueEdit5->text());
    query.bindValue(":customLabel6", customLabelEdit6->text());
    query.bindValue(":customValue6", customValueEdit6->text());
    query.bindValue(":id", *selectedContactId);
    query.exec();
}


bool AddressWidget::addContactToDB()
{
    QSqlQuery query(*db);

    /* Check if already exists */
    query.prepare("SELECT * FROM contacts WHERE name=:name AND firstName=:firstName");
    query.bindValue(":name", nameLineEdit->text());
    query.bindValue(":firstName", firstNameLineEdit->text());
    query.exec();
    query.next(); // avoid error
    if (!query.isNull(0)) {
        QMessageBox::warning(this, tr("Database Warning"),
                             tr("This contact already exists in your database"));
        return false;
    }


    if (mainWindow->accountType == "network") {

        if (!mainWindow->isRevUpToDate())
            return false;

        if (!mainWindow->connectToServer())
            return false;

        mainWindow->statusBar()->showMessage(tr("Sending new contact to server..."));

        QByteArray newContact;
        QDataStream outData(&newContact, QIODevice::WriteOnly);
        outData.setVersion(QDataStream::Qt_4_5);

        outData << nameLineEdit->text() << firstNameLineEdit->text() << workLineEdit->text()
                << homeLineEdit->text() << mobileLineEdit->text() << addressLineEdit->text()
                << zipLineEdit->text() << townLineEdit->text() << email1LineEdit->text()
                << webLineEdit->text() << noteTextEdit->toPlainText() << (*customCount)
                << customLabelEdit1->text() << customValueEdit1->text() << customLabelEdit2->text()
                << customValueEdit2->text() << customLabelEdit3->text() << customValueEdit3->text()
                << customLabelEdit4->text() << customValueEdit4->text() << customLabelEdit5->text()
                << customValueEdit5->text() << customLabelEdit6->text() << customValueEdit6->text();

        QByteArray outBlock;
        QDataStream out(&outBlock, QIODevice::WriteOnly);
        out.setVersion(QDataStream::Qt_4_5);
        out << qint64(0);
        out << (*mainWindow->localServerPass) << quint16(ADD_REQUEST) << newContact;
        out.device()->seek(0);
        out << qint64(outBlock.size() - sizeof(qint64));

        mainWindow->statusBar()->showMessage(tr("Sending ADD request to server..."));

        mainWindow->tcpSocket->write(outBlock);

        if (!mainWindow->tcpSocket->waitForBytesWritten()) {
            mainWindow->statusBar()->showMessage(tr("Server refused ADD request!"));
            return false;
        }

        if (!mainWindow->tcpSocket->waitForReadyRead())
            return false;

        QDataStream in(mainWindow->tcpSocket);
        in.setVersion(QDataStream::Qt_4_5);

        /* Kontakter packet format */
        qint64 size;
        QString serverPass;
        quint16 reqType;
        QByteArray data;

        in >> size >> serverPass >> reqType >> data;

        if (reqType == ADD_REQUEST) {

            mainWindow->statusBar()->showMessage(tr("Contact added successfully!"));

        } else if (reqType == ERROR_REPORT) {

            QMessageBox::critical(this, tr("Error"), QString(data));
            mainWindow->statusBar()->showMessage(tr("Error!"));
            mainWindow->disconnectFromServer();
            return false;

        } else {
            mainWindow->statusBar()->showMessage(tr("Request Type Error!"));
            mainWindow->disconnectFromServer();
            return false;
        }

        mainWindow->incrementLocalRevision();

        mainWindow->disconnectFromServer();
    }


    /* Save to local DB */
    query.prepare(
            "INSERT INTO contacts (name, firstName, work, home, mobile, address, zip, town, email, web, notes, customCount, customLabel1, customValue1, customLabel2, customValue2, customLabel3, customValue3, customLabel4, customValue4, customLabel5, customValue5, customLabel6, customValue6, date) "
            "VALUES (:name, :firstName, :work, :home, :mobile, :address, :zip, :town, :email, :web, :notes, :customCount, :customLabel1, :customValue1, :customLabel2, :customValue2, :customLabel3, :customValue3, :customLabel4, :customValue4, :customLabel5, :customValue5, :customLabel6, :customValue6, CURRENT_TIMESTAMP)"
            );
    query.bindValue(":name", nameLineEdit->text());
    query.bindValue(":firstName", firstNameLineEdit->text());
    query.bindValue(":work", workLineEdit->text());
    query.bindValue(":home", homeLineEdit->text());
    query.bindValue(":mobile", mobileLineEdit->text());
    query.bindValue(":address", addressLineEdit->text());
    query.bindValue(":zip", zipLineEdit->text());
    query.bindValue(":town", townLineEdit->text());
    query.bindValue(":email", email1LineEdit->text());
    query.bindValue(":web", webLineEdit->text());
    query.bindValue(":notes", noteTextEdit->toPlainText());
    query.bindValue(":customCount", *customCount);
    query.bindValue(":customLabel1", customLabelEdit1->text());
    query.bindValue(":customValue1", customValueEdit1->text());
    query.bindValue(":customLabel2", customLabelEdit2->text());
    query.bindValue(":customValue2", customValueEdit2->text());
    query.bindValue(":customLabel3", customLabelEdit3->text());
    query.bindValue(":customValue3", customValueEdit3->text());
    query.bindValue(":customLabel4", customLabelEdit4->text());
    query.bindValue(":customValue4", customValueEdit4->text());
    query.bindValue(":customLabel5", customLabelEdit5->text());
    query.bindValue(":customValue5", customValueEdit5->text());
    query.bindValue(":customLabel6", customLabelEdit6->text());
    query.bindValue(":customValue6", customValueEdit6->text());
    query.exec();

    return true;
}

void AddressWidget::checkPass()
{
    passWidget = new PassWidget;
    connect(passWidget->validateButton, SIGNAL(clicked()),
            this, SLOT(checkPassSlot()));
    connect(passWidget->cancelButton, SIGNAL(clicked()),
            this, SLOT(passCancelSlot()));
    passWidget->show();
}

/* _________________________________________________________________________ */
