/*
 *  Copyright (C) 2009  Giorgio Wicklein <g.wicklein@giowisys.com> 
 *  
 *  This file is part of Kontakter.
 *
 *  Kontakter is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Kontakter is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Kontakter.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QtGui>
#include <QtNetwork>

#include "mainwindow.h"
#include "newAccountWidget/newaccountwidget.h"
#include "mainwidget.h"
#include "addresswidget.h"
#include "contactlistwidget.h"
#include "searchwidget.h"
#include "settingswidget.h"
#include "printwidget.h"
#include "aboutdialog.h"
#include "definitions.h"

/* _________________________________PUBLIC__________________________________ */

//// constructor
MainWindow::MainWindow()
{
    /****** VERSION STUFF *******/
    buildNumber = new int;
    *buildNumber = 3;
    version = new QString;
    *version = " v0.2";
    /***************************/
    
    currentDatabase = new QString;
    accountsActionList = new QList<QAction*>;
    mainWidget = new MainWidget(this);
    http = new QHttp(this);

    createActions();
    createMenus();
    createToolBar();
    createStatusBar();
    createTrayIcon();
    createConnections();
    settingsWidget = new SettingsWidget(this);
    printWidget = new PrintWidget(this);

    stackedWidget = new QStackedWidget(this);
    stackedWidget->addWidget(mainWidget);
    stackedWidget->addWidget(settingsWidget);
    stackedWidget->addWidget(printWidget);

    mainLayout = new QVBoxLayout;
    mainLayout->addWidget(stackedWidget);

    setCentralWidget(stackedWidget);

    setWindowTitle(tr("Kontakter"));

    settingsWidgetActive = new bool;
    *settingsWidgetActive = false;
    printWidgetActive = new bool;
    *printWidgetActive = false;
    localServerPass = new QString();

    updateActions();
    readSettings();
    checkFirstRun();
    updateAccountsMenu();
    updateActions();
    checkUpdates();
    checkDate();


    /* socket stuff */
    tcpSocket = new QTcpSocket(this);

    connect(tcpSocket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(socketError()));

    loadAccountSettings();
    updateActions();
}

void MainWindow::disableAll(bool d)
{
    d = !d;

    newContactAction->setEnabled(d);
    editContactAction->setEnabled(d);
    deleteContactAction->setEnabled(d);
    printAction->setEnabled(d);
    settingsAction->setEnabled(d);

    /* syncContactAction */
    if (!d) {
        QSettings settings("giowisys", "Kontakter");
        settings.beginGroup(*currentDatabase);
        QString type = settings.value("type").toString();
        settings.endGroup();

        if (type == "network")
            syncContactsAction->setEnabled(true);
        else
            syncContactsAction->setEnabled(false);
    } else {
        syncContactsAction->setEnabled(false);
    }

    deleteAccountAction->setEnabled(d);
    exportMenu->setEnabled(d);
    importMenu->setEnabled(d);
}

void MainWindow::disableSyncAction()
{
    syncContactsAction->setEnabled(false);
}

bool MainWindow::connectToServer()
{   
    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup(*currentDatabase);
    QString host = settings.value("host", "127.0.0.1").toString();
    int port = settings.value("port", 1337).toInt();
    settings.endGroup();

    QProgressDialog progress(tr("Connecting to the server..."), tr("Please wait"), 0, 0, this, Qt::Sheet);
    progress.show();

    statusBar()->showMessage(tr("Connecting to the server..."));

    qApp->processEvents();

    tcpSocket->connectToHost(host, port);

    qApp->processEvents();

    if (tcpSocket->waitForConnected(10000)) {
        statusBar()->showMessage(tr("Connected successfully!"));
    } else {
        QMessageBox::critical(this, tr("Connection error"), tr("Cannot connect to the server! "
                     "Make sure the server is running and if your computer is behind a router"
                     " check that the current port is forwarded correctly!"));
        return false;
    }

    return true;
}

void MainWindow::disconnectFromServer()
{
    tcpSocket->disconnectFromHost();
    tcpSocket->waitForDisconnected();
}

bool MainWindow::isRevUpToDate()
{
    if (!connectToServer())
            return false;

    QByteArray outBlock;
    QDataStream out(&outBlock, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_5);
    out << qint64(0);
    out << *localServerPass << quint16(REVISION_REQUEST) << QByteArray(QString::number(localRevision).toStdString().c_str());
    out.device()->seek(0);
    out << qint64(outBlock.size() - sizeof(qint64));

    statusBar()->showMessage(tr("Sending REVISION request to server..."));

    tcpSocket->write(outBlock);

    if (!tcpSocket->waitForBytesWritten()) {
        statusBar()->showMessage(tr("Server refused REVISION request!"));
        return false;
    }

    if (!tcpSocket->waitForReadyRead())
        return false;

    QDataStream in(tcpSocket);
    in.setVersion(QDataStream::Qt_4_5);

    /* Kontakter packet format */
    qint64 size;
    QString serverPass;
    quint16 reqType;
    QByteArray data;

    in >> size;

    while (tcpSocket->bytesAvailable() < size) {
        if (!tcpSocket->waitForReadyRead()) {
            qDebug() << "Not enough bytes!";
            disconnectFromServer();
            return false;
        }
    }

    in >> serverPass >> reqType >> data;

    if (reqType == REVISION_REQUEST) {

        quint64 serverRevision =  QString(data).toULongLong();

        if (serverRevision != localRevision) {
            QMessageBox::warning(this, tr("Database not up to date"),
                                 tr("Database is not updated! Please synchronize your contacts with the server"));
            return false;
        }

    } else if(reqType == ERROR_REPORT) {

        QMessageBox::critical(this, tr("Error"), QString(data));
        statusBar()->showMessage(tr("Error!"));
        disconnectFromServer();
        return false;

    } else {
        statusBar()->showMessage(tr("Request Type Error!"));
        disconnectFromServer();
        return false;
    }

    statusBar()->showMessage(tr("Database is up to date!"));
    disconnectFromServer();
    return true;
}

void MainWindow::incrementLocalRevision()
{
    ++localRevision;

    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup(*currentDatabase);

    settings.setValue("localRevision", localRevision);

    settings.endGroup();
}

/* _________________________________________________________________________ */
/* ______________________________PUBLIC SLOTS_______________________________ */

void MainWindow::updateActions()
{
    if ((mainWidget->contactListWidget->listWidget->count() < 1)
        || *settingsWidgetActive
        || *printWidgetActive) {
        editContactAction->setEnabled(false);
        deleteContactAction->setEnabled(false);
    }
    if (mainWidget->contactListWidget->listWidget->selectedItems().isEmpty()) {
        editContactAction->setEnabled(false);
        deleteContactAction->setEnabled(false);
    } else {
        editContactAction->setEnabled(true);
        deleteContactAction->setEnabled(true);
    }
    if ((*settingsWidgetActive) || (*printWidgetActive))
        newContactAction->setEnabled(false);
    else {
        int dbCount = 0;
        QSettings settings("giowisys", "Kontakter");

        settings.beginGroup("database");
        dbCount = settings.value("dbCount").toInt();
        settings.endGroup();

        if (dbCount) {
            newContactAction->setEnabled(true);
        } else
            newContactAction->setEnabled(false);
    }
}

void MainWindow::newAccountCancSlot()
{
    if (*firstRun)
        qApp->quit();
    else
        delete newAccountWidget;
}

/* _________________________________________________________________________ */
/* _________________________________EVENTS__________________________________ */

void MainWindow::closeEvent(QCloseEvent *event)
{
    if (isVisible()) {
        hide();
        trayIcon->showMessage(tr("Kontakter"), tr("Running..."),
                              QSystemTrayIcon::NoIcon);
        event->ignore();
    }
    else {
        writeSettings();
        event->accept();
    }
}

/* _________________________________________________________________________ */
/* ______________________________PRIVATE SLOTS______________________________ */

void MainWindow::handleEditModeSlot(bool activated)
{
    activated = !activated;
    newContactAction->setEnabled(activated);
    editContactAction->setEnabled(activated);
    deleteContactAction->setEnabled(activated);
}

void MainWindow::handleQuitSlot()
{
    if (accountType == "network")
        goodbyeServer();

    writeSettings();
    qApp->quit();
}

void MainWindow::settingsActionSlot()
{
    if (*printWidgetActive) {
        *printWidgetActive = false;
        settingsActionSlot();
    } else if (*settingsWidgetActive) {
        settingsWidget->saveSettingsSlot();
        stackedWidget->setCurrentIndex(0);
        *settingsWidgetActive = false;
        *printWidgetActive = false;
    } else {
        stackedWidget->setCurrentIndex(1);
        *settingsWidgetActive = true;
    }
    updateActions();
}

void MainWindow::printSlot()
{
    if (printWidget->selectedRadioButton->isChecked()
        &&
        !(mainWidget->contactListWidget->listWidget->selectedItems().isEmpty())) {
        mainWidget->addressWidget->print(1, false);
    }
    else if (printWidget->allRadioButton->isChecked()) {
        mainWidget->addressWidget->print(2, false);
    }
    else {
        QMessageBox::warning(this, tr("Print warning"),
                             tr("Print aborted:\nNo contact selected!"));
        return;
    }

    printActionSlot();
}

void MainWindow::printActionSlot()
{
    if (*settingsWidgetActive) {
        *settingsWidgetActive = false;
        printActionSlot();
    } else if (*printWidgetActive || *settingsWidgetActive) {
        stackedWidget->setCurrentIndex(0);
        *printWidgetActive = false;
        *settingsWidgetActive = false;
    } else {
        stackedWidget->setCurrentIndex(2);
        *printWidgetActive = true;

        if (mainWidget->contactListWidget->listWidget->selectedItems().isEmpty()) {
            printWidget->contactName->setText(tr("None Selected"));
        } else {
            printWidget->contactName->setText(
                    mainWidget->contactListWidget->listWidget->currentItem()->text()
                    );
        }
    }
    updateActions();
}

void MainWindow::trayActivated(QSystemTrayIcon::ActivationReason reason)
{
    switch (reason) {
        case QSystemTrayIcon::DoubleClick:
            if (isHidden())
                show();
            else
                hide();
            break;
        default:
            ;
     }
}

void MainWindow::newAccountSlot()
{
    newAccountWidget = new NewAccountWidget;
    connect(newAccountWidget, SIGNAL(completeSignal()),
            this, SLOT(newAccountCompleteSlot()));
    connect(newAccountWidget, SIGNAL(cancelNewAccSlot()),
            this, SLOT(newAccountCancSlot()));
    newAccountWidget->show();
}

void MainWindow::newAccountCompleteSlot()
{
    delete newAccountWidget;
    mainWidget->addressWidget->prepareContactsDB();
    mainWidget->addressWidget->loadContactsDB();
    updateAccountsMenu();
    loadAccountSettings();
    this->show();
    updateActions();
}

void MainWindow::deleteAccountSlot()
{
    int r = QMessageBox::warning(this, tr("Delete account"),
                             tr("Are you sure you want to delete"
                                " %1?").arg(*currentDatabase),
                             QMessageBox::No | QMessageBox::Yes);

    if (r == QMessageBox::No)
        return;

    mainWidget->addressWidget->db->close();
    mainWidget->addressWidget->db->removeDatabase("QSQLITE");

    QString fullPath = QDir::homePath()
                       + "/.Kontakter/databases/"
                       + (*currentDatabase)
                       + ".sqlite";
    QDir dir;
    dir.remove(fullPath);

    QStringList databases;

    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup("database");
    int dbCount = settings.value("dbCount").toInt();
    --dbCount;
    settings.setValue("dbCount", dbCount);
    databases = settings.value("databases").toStringList();
    databases.removeOne(*currentDatabase);
    settings.setValue("databases", databases);

    if (!databases.isEmpty())
        settings.setValue("currentDatabase", databases.at(0));

    settings.endGroup();

    settings.remove(*currentDatabase);

    settings.beginGroup("database");
    databases = settings.value("databases").toStringList();
    settings.endGroup();

    if (!databases.isEmpty()) {
        *currentDatabase = databases.at(0);
        mainWidget->addressWidget->prepareContactsDB();
        mainWidget->addressWidget->loadContactsDB();
        loadAccountSettings();
    } else {
        mainWidget->contactListWidget->listWidget->clear();
        accountType = "local";
    }


    updateAccountsMenu();
    updateActions();
    stackedWidget->setCurrentIndex(0);
}

void MainWindow::loadAccountSettings()
{
    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup(*currentDatabase);
    QString type = settings.value("type").toString();
    mainWidget->addressWidget->clearContactWidget();

    if (type == "network") {
        syncContactsAction->setEnabled(true);
        accountType = "network";
        localRevision = settings.value("localRevision", 0).toULongLong();
        *localServerPass = settings.value("serverPass").toString();
        importDatabaseAction->setEnabled(false);

        helloServer();
    }
    else {
        syncContactsAction->setEnabled(false);
        accountType = "local";
        importDatabaseAction->setEnabled(true);
    }

    settings.endGroup();
    settingsWidget->loadSettings();
}

void MainWindow::accountChangeSlot()
{
    if (accountType == "network")
        goodbyeServer();

    *mainWidget->addressWidget->validPass = false;
    mainWidget->searchWidget->searchLineEdit->clear();

    QString selectedAccount = accountActionGroup->checkedAction()->text();
    *currentDatabase = selectedAccount;

    mainWidget->addressWidget->db->close();
    mainWidget->addressWidget->db->removeDatabase("QSQLITE");
    delete mainWidget->addressWidget->db;

    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup("database");
    settings.setValue("currentDatabase", selectedAccount);
    settings.endGroup();

    bool prepareDB = mainWidget->addressWidget->prepareContactsDB();
    bool loadDB = false;

    if (prepareDB)
        loadDB = mainWidget->addressWidget->loadContactsDB();

    mainWidget->searchWidget->searchLineEdit->setFocus();

    disableAll(false);

    loadAccountSettings();

    updateActions();

    stackedWidget->setCurrentIndex(0);
}

void MainWindow::exportSelectedPdf()
{
    if (mainWidget->contactListWidget->listWidget->selectedItems().isEmpty()) {
        QMessageBox::warning(this, tr("Export warning"),
                             tr("Export aborted:\nNo contact selected!"));
        return;
    }

    mainWidget->addressWidget->print(1, true);
}

void MainWindow::exportAllPdf()
{
    mainWidget->addressWidget->print(2, true);
}

void MainWindow::exportDatabase()
{
    QString outFile =
                QFileDialog::getSaveFileName(this, tr("Export database file"),
                                             QDir::homePath(),
                                             tr("SQLITE files (*.sqlite)"));
    if (outFile.isEmpty())
            return;

    if (!outFile.contains(".sqlite"))
            outFile.append(".sqlite");

    QFile fileDb(QDir::homePath()
                       + "/.Kontakter/databases/"
                       + (*currentDatabase)
                       + ".sqlite");

    bool cp = fileDb.copy(outFile);

    if (!cp) {
        QMessageBox::critical(this, tr("Error in database export"),
                              tr("An error occurred while exporting the database!"));
        return;
    }
    statusBar()->showMessage(tr("Database exported successfully!"));
}

void MainWindow::importDatabase()
{
    QString inFile = QFileDialog::getOpenFileName(this, tr("Import database file"),
                                             QDir::homePath(),
                                             tr("SQLITE files (*.sqlite)"));

    if (inFile.isEmpty())
        return;

    bool r = QMessageBox::warning(this, tr("Import database"),
                                  tr("Are you sure you want to substitute the "
                                     "current database with the external one?"),
                                  QMessageBox::No | QMessageBox::Yes);

    if (r == QMessageBox::No)
        return;

    mainWidget->addressWidget->db->close();
    mainWidget->addressWidget->db->removeDatabase("QSQLITE");
    delete mainWidget->addressWidget->db;

    QString fullPath = QDir::homePath()
                       + "/.Kontakter/databases/"
                       + (*currentDatabase)
                       + ".sqlite";
    QDir dir;

    if (!dir.remove(fullPath)) {
        QMessageBox::critical(this, tr("File error"),
                              tr("Cannot delete the database!"));
        return;
    }

    if (!QFile::copy(inFile, fullPath)) {
        QMessageBox::critical(this, tr("File error"),
                              tr("Cannot write the database!"));
    }

    mainWidget->addressWidget->prepareContactsDB();
    mainWidget->addressWidget->loadContactsDB();
    mainWidget->searchWidget->searchLineEdit->setFocus();

    updateActions();
    statusBar()->showMessage(tr("Database imported successfully!"));
}

void MainWindow::exportvCardSelSlot()
{
    if (mainWidget->contactListWidget->listWidget->selectedItems().isEmpty()) {
        QMessageBox::warning(this, tr("Export warning"),
                             tr("Export aborted:\nNo contact selected!"));
        return;
    }

    mainWidget->addressWidget->exportvCard(false);
}

void MainWindow::exportvCardAllSlot()
{
    mainWidget->addressWidget->exportvCard(true);
}

void MainWindow::updateSlot()
{
    QUrl url("http://update.giowisys.com/kontakter/updates");

    http->setHost(url.host(), url.port(80));
    http->get(url.path());
}

void MainWindow::updateDoneSlot(bool error)
{
    if (error) {
        statusBar()->showMessage(tr("An error occurred while checking for updates..."));
        return;
    }

    int lastBuild = QString(http->readAll()).toInt();

    if (lastBuild > (*buildNumber)) {
        int r = QMessageBox::question(this, tr("Kontakter update"),
                                 tr("New software version available!\n"
                                    "Do you want to download the new version?"),
                                 QMessageBox::Yes | QMessageBox::No);

        statusBar()->showMessage(tr("Download the new software version at http://giowisys.com"));

        if (r == QMessageBox::Yes)
            QDesktopServices::openUrl(QUrl("http://giowisys.com/products/kontakter/"));

    } else {
        statusBar()->showMessage(tr("No software updates found!"));
    }
}

void MainWindow::documentationSlot()
{
    QDesktopServices::openUrl(QUrl("http://giowisys.com/products/kontakter/doc/"));
}

void MainWindow::aboutSlot()
{
    AboutDialog *aboutDialog = new AboutDialog(this);
    aboutDialog->deleteLater();
    aboutDialog->exec();
}

void MainWindow::reportBugSlot()
{
    QDesktopServices::openUrl(QUrl("https://bugs.launchpad.net/kontakter"));
}

void MainWindow::askSlot()
{
    QDesktopServices::openUrl(QUrl("https://answers.launchpad.net/kontakter"));
}

void MainWindow::featureRequestSlot()
{
    QDesktopServices::openUrl(QUrl("https://blueprints.launchpad.net/kontakter"));
}

void MainWindow::socketError()
{
    qDebug() << "Socket Error: " << tcpSocket->errorString();
}

void MainWindow::syncDatabase()
{
    if (!connectToServer())
            return;

    QByteArray outBlock;
    QDataStream out(&outBlock, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_5);
    out << qint64(0);
    out << *localServerPass << quint16(REVISION_REQUEST) << QByteArray(QString::number(localRevision).toStdString().c_str());
    out.device()->seek(0);
    out << qint64(outBlock.size() - sizeof(qint64));

    statusBar()->showMessage(tr("Sending REVISION request to server..."));

    tcpSocket->write(outBlock);

    if (!tcpSocket->waitForBytesWritten()) {
        statusBar()->showMessage(tr("Server refused REVISION request!"));
        return;
    }

    if (!tcpSocket->waitForReadyRead())
        return;

    QDataStream in(tcpSocket);
    in.setVersion(QDataStream::Qt_4_5);

    /* Kontakter packet format */
    qint64 size;
    QString serverPass;
    quint16 reqType;
    QByteArray data;

    quint64 serverRevision;

    in >> size;

    while (tcpSocket->bytesAvailable() < size) {
        if (!tcpSocket->waitForReadyRead()) {
            qDebug() << "Not enough bytes!";
            disconnectFromServer();
            return;
        }
    }

    in >> serverPass >> reqType >> data;

    if (reqType == REVISION_REQUEST) {

        serverRevision =  QString(data).toULongLong();

        if (serverRevision == localRevision) {
            statusBar()->showMessage(tr("Database is already up to date!"));
            return;
        }

    } else if(reqType == ERROR_REPORT) {

        QMessageBox::critical(this, tr("Error"), QString(data));
        statusBar()->showMessage(tr("Error!"));
        disconnectFromServer();
        return;

    } else {
        statusBar()->showMessage(tr("Request Type Error!"));
        disconnectFromServer();
        return;
    }

    disconnectFromServer();

    if (!connectToServer())
        return;

    statusBar()->showMessage(tr("Synchronizing the database..."));

    /* Get the full database */
    QByteArray outBlock2;
    QDataStream out2(&outBlock2, QIODevice::WriteOnly);
    out2.setVersion(QDataStream::Qt_4_5);
    out2 << qint64(0);
    out2 << *localServerPass << quint16(GET_DB) << QByteArray();
    out2.device()->seek(0);
    out2 << qint64(outBlock2.size() - sizeof(qint64));

    tcpSocket->write(outBlock2);

    if (!tcpSocket->waitForBytesWritten()) {
        statusBar()->showMessage(tr("Server refused SYNC request!"));
        return;
    }

    if (!tcpSocket->waitForReadyRead())
        return;

    QDataStream inStream(tcpSocket);
    inStream.setVersion(QDataStream::Qt_4_5);

    /* Kontakter packet format */
    qint64 size2;
    QString serverPass2;
    quint16 reqType2;
    QByteArray data2;

    inStream >> size2;

    while (tcpSocket->bytesAvailable() < size2) {
        if (!tcpSocket->waitForReadyRead()) {
            qDebug() << "Not enough bytes!";
            disconnectFromServer();
            return;
        }
    }

    inStream >> serverPass2 >> reqType2 >> data2;

    if (reqType2 != GET_DB) {
        return;
    }

    if (!data2.size()) {
        QMessageBox::critical(this, tr("Invalid database"), tr("Could not recive the database from the server!"));
        disconnectFromServer();
        return;
    }

    QString dbName = QDir::homePath() + "/.Kontakter/databases/"
                     + (*currentDatabase)
                     + ".sqlite";

    mainWidget->addressWidget->db->close();
    mainWidget->addressWidget->db->removeDatabase("QSQLITE");

    QDir dir;
    if (!dir.remove(dbName)) {
        QMessageBox::critical(this, tr("File error"),
                              tr("Cannot delete the old database!"));
        disconnectFromServer();
        return;
    }

    QFile outDb(dbName);

    if (!outDb.open(QIODevice::WriteOnly)) {
        QMessageBox::critical(this, tr("File error"),
                              tr("Cannot create new database!"));
        disconnectFromServer();
        return;
    }

    if (outDb.write(data2) < 1) {
        QMessageBox::critical(this, tr("File error"),
                              tr("Cannot write database file!\nRicived database size: %1").arg(data2.size()));
        disconnectFromServer();
        return;
    }

    outDb.close();

    mainWidget->addressWidget->prepareContactsDB();
    mainWidget->addressWidget->loadContactsDB();
    mainWidget->searchWidget->searchLineEdit->setFocus();

    updateActions();

    disconnectFromServer();

    statusBar()->showMessage(tr("Database synchronized successfully!"));

    /* update localRevision */
    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup(*currentDatabase);

    settings.setValue("localRevision", serverRevision);
    localRevision = serverRevision;

    settings.endGroup();
}

/* _________________________________________________________________________ */
/* _________________________________PRIVATE_________________________________ */

void MainWindow::createActions()
{
    newContactAction = new QAction(tr("&New Contact"), this);
    newContactAction->setShortcut(QKeySequence::New);
    newContactAction->setStatusTip(tr("Add a new contact"));
    newContactAction->setIcon(QIcon(":/icons/add_contact.png"));

    editContactAction = new QAction(tr("&Edit Contact"), this);
    editContactAction->setStatusTip(tr("Edit selected contact"));
    editContactAction->setIcon(QIcon(":/icons/edit_contact.png"));

    deleteContactAction = new QAction(tr("&Delete Contact"), this);
    deleteContactAction->setShortcut(QKeySequence::Delete);
    deleteContactAction->setStatusTip(tr("Delete the selected contact"));
    deleteContactAction->setIcon(QIcon(":/icons/delete_contact.png"));

    syncContactsAction = new QAction(tr("Syn&c Contacts"), this);
    syncContactsAction->setShortcut(tr("CTRL+C"));
    syncContactsAction->setStatusTip(
            tr("Synchronize your contacts with the central database"));
    syncContactsAction->setIcon(QIcon(":/icons/network.png"));
    connect(syncContactsAction, SIGNAL(triggered()), this, SLOT(syncDatabase()));

    settingsAction = new QAction(tr("&Settings..."), this);
    settingsAction->setStatusTip(tr("Settings..."));
    settingsAction->setIcon(QIcon(":/icons/settings.png"));
    connect(settingsAction, SIGNAL(triggered()),
            this, SLOT(settingsActionSlot()));

    printAction = new QAction(tr("&Print..."), this);
    printAction->setShortcut(QKeySequence::Print);
    printAction->setStatusTip(tr("Print only selected or all contacts"));
    printAction->setIcon(QIcon(":/icons/print.png"));
    connect(printAction, SIGNAL(triggered()),
            this, SLOT(printActionSlot()));

    exitAction = new QAction(tr("E&xit"), this);
    exitAction->setShortcut(tr("CTRL+Q"));
    exitAction->setStatusTip(tr("Exit the application"));
    exitAction->setIcon(QIcon(":/icons/exit.png"));
    connect(exitAction, SIGNAL(triggered()), this, SLOT(handleQuitSlot()));

    newAccountAction = new QAction(tr("New account"), this);
    newAccountAction->setStatusTip(tr("Create a new account"));
    connect(newAccountAction, SIGNAL(triggered()),
            this, SLOT(newAccountSlot()));

    deleteAccountAction = new QAction(tr("Delete account"), this);
    deleteAccountAction->setStatusTip(tr("Delete current account"));
    deleteAccountAction->setEnabled(false);
    connect(deleteAccountAction, SIGNAL(triggered()),
            this, SLOT(deleteAccountSlot()));

    accountActionGroup = new QActionGroup(this);

    exportSelectedPdfAction = new QAction(tr("Selected contact"), this);
    exportSelectedPdfAction->setStatusTip(tr("Export the selected contact to PDF file"));
    connect(exportSelectedPdfAction, SIGNAL(triggered()), this, SLOT(exportSelectedPdf()));

    exportAllPdfAction = new QAction(tr("All contacts"), this);
    exportAllPdfAction->setStatusTip(tr("Export all contacts to PDF file"));
    connect(exportAllPdfAction, SIGNAL(triggered()), this, SLOT(exportAllPdf()));

    exportDatabaseAction = new QAction(tr("Database"), this);
    exportDatabaseAction->setStatusTip(tr("Export the database, including all contacts"));
    connect(exportDatabaseAction, SIGNAL(triggered()), this, SLOT(exportDatabase()));

    exportvCardSelAction = new QAction(tr("Selected contact"), this);
    exportvCardSelAction->setStatusTip(tr("Export selected contact as Business Card (vCard)"));
    connect(exportvCardSelAction, SIGNAL(triggered()), this, SLOT(exportvCardSelSlot()));

    exportvCardAllAction = new QAction(tr("All contacts"), this);
    exportvCardAllAction->setStatusTip(tr("Export all contacts as Business Card (vCard)"));
    connect(exportvCardAllAction, SIGNAL(triggered()), this, SLOT(exportvCardAllSlot()));

    importvCardAction = new QAction(tr("Business Card (vCard)"), this);
    importvCardAction->setStatusTip(tr("Import business cards (vCard)"));
    connect(importvCardAction, SIGNAL(triggered()),
            mainWidget->addressWidget, SLOT(importvCard()));

    importDatabaseAction = new QAction(tr("Database"), this);
    importDatabaseAction->setStatusTip(tr("Import a database, including all contacts"));
    connect(importDatabaseAction, SIGNAL(triggered()), this, SLOT(importDatabase()));

    aboutQtAction = new QAction(tr("About Qt"), this);
    connect(aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

    aboutAction = new QAction(tr("About"), this);
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(aboutSlot()));

    documentationAction = new QAction(tr("Online documentation"), this);
    documentationAction->setStatusTip(tr("Browse the online documentation"));
    connect(documentationAction, SIGNAL(triggered()), this, SLOT(documentationSlot()));

    updateAction = new QAction(tr("Check for updates"), this);
    updateAction->setStatusTip(tr("Check for software updates..."));
    connect(updateAction, SIGNAL(triggered()), this, SLOT(updateSlot()));

    reportBugAction = new QAction(tr("Report a bug"), this);
    reportBugAction->setStatusTip(tr("Improve Kontakter by reporting bugs"));
    connect(reportBugAction, SIGNAL(triggered()), this, SLOT(reportBugSlot()));

    askAction = new QAction(tr("Ask a question"), this);
    askAction->setStatusTip(tr("Ask a question about Kontakter"));
    connect(askAction, SIGNAL(triggered()), this, SLOT(askSlot()));

    featureRequestAction = new QAction(tr("Request a feature"), this);
    featureRequestAction->setStatusTip(tr("Improve Kontaker with your idea"));
    connect(featureRequestAction, SIGNAL(triggered()), this, SLOT(featureRequestSlot()));
}

void MainWindow::createMenus()
{
    pdfMenu = new QMenu(tr("P&DF"), this);
    pdfMenu->addAction(exportSelectedPdfAction);
    pdfMenu->addAction(exportAllPdfAction);

    vCardMenu = new QMenu(tr("Business Card (vCard)"), this);
    vCardMenu->addAction(exportvCardSelAction);
    vCardMenu->addAction(exportvCardAllAction);

    exportMenu = new QMenu(tr("&Export"), this);
    exportMenu->addMenu(pdfMenu);
    exportMenu->addMenu(vCardMenu);
    exportMenu->addAction(exportDatabaseAction);

    importMenu = new QMenu(tr("&Import"), this);
    importMenu->addAction(importvCardAction);
    importMenu->addAction(importDatabaseAction);

    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(newContactAction);
    fileMenu->addSeparator();
    fileMenu->addAction(syncContactsAction);
    fileMenu->addAction(printAction);
    fileMenu->addSeparator();
    fileMenu->addMenu(importMenu);
    fileMenu->addMenu(exportMenu);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAction);

    editMenu = menuBar()->addMenu(tr("&Edit"));
    editMenu->addAction(editContactAction);
    editMenu->addAction(deleteContactAction);
    editMenu->addSeparator();
    editMenu->addAction(settingsAction);

    accountsMenu = menuBar()->addMenu(tr("&Accounts"));
    accountsMenu->addAction(newAccountAction);
    accountsMenu->addAction(deleteAccountAction);
    accountsMenu->addSeparator();

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAction);
    helpMenu->addAction(aboutQtAction);
    helpMenu->addAction(documentationAction);
    helpMenu->addSeparator();
    helpMenu->addAction(reportBugAction);
    helpMenu->addAction(askAction);
    helpMenu->addAction(featureRequestAction);
    helpMenu->addSeparator();
    helpMenu->addAction(updateAction);
}

void MainWindow::createToolBar()
{
    toolBar = new QToolBar(tr("Menu"), this);
#ifndef Q_WS_MAC
    addToolBar(Qt::LeftToolBarArea, toolBar);
#endif
#ifdef Q_WS_MAC
    setUnifiedTitleAndToolBarOnMac(true);
    addToolBar(toolBar);
#endif
    toolBar->setMovable(false);
    toolBar->addAction(newContactAction);
    toolBar->addAction(editContactAction);
    toolBar->addAction(deleteContactAction);
    toolBar->addSeparator();
    toolBar->addAction(syncContactsAction);
    toolBar->addAction(settingsAction);
    toolBar->addAction(printAction);
    toolBar->addSeparator();
    toolBar->addAction(exitAction);
}

void MainWindow::createStatusBar()
{
    statusBar()->showMessage(tr("Ready"));
}

void MainWindow::createTrayIcon()
{
    showAction = new QAction(tr("Show Kontakter"), this);
    connect(showAction, SIGNAL(triggered()), this, SLOT(show()));
    hideAction = new QAction(tr("Hide Kontakter"), this);
    connect(hideAction, SIGNAL(triggered()), this, SLOT(hide()));

    trayMenu = new QMenu(this);
    trayMenu->addAction(showAction);
    trayMenu->addAction(hideAction);
    trayMenu->addSeparator();
    trayMenu->addAction(exitAction);

    trayIcon = new QSystemTrayIcon(this);
    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
             this, SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));
    trayIcon->setContextMenu(trayMenu);
    trayIcon->setIcon(QIcon(":/icons/kontakter.png"));
    trayIcon->setToolTip(tr("Kontakter"));
    trayIcon->show();
}

void MainWindow::createConnections()
{
    connect(newContactAction, SIGNAL(triggered()), mainWidget->addressWidget,
            SLOT(addContact()));
    connect(editContactAction, SIGNAL(triggered()), mainWidget->addressWidget,
            SLOT(editContactSlot()));
    connect(mainWidget->addressWidget, SIGNAL(updateActionsSignal()),
            this, SLOT(updateActions()));
    connect(deleteContactAction, SIGNAL(triggered()), mainWidget->addressWidget,
            SLOT(deleteContactSlot()));
    connect(mainWidget->addressWidget, SIGNAL(editModeSignal(bool)),
            this, SLOT(handleEditModeSlot(bool)));
    connect(mainWidget->addressWidget, SIGNAL(addContactListSignal(QString)),
            mainWidget->contactListWidget, SLOT(addContact(QString)));
    connect(mainWidget->searchWidget->searchLineEdit, SIGNAL(textChanged(QString)),
            mainWidget->addressWidget, SLOT(searchSlot(QString)));
    connect(http, SIGNAL(done(bool)), this, SLOT(updateDoneSlot(bool)));
}

void MainWindow::writeSettings()
{
    QSettings settings("giowisys", "Kontakter");

    settings.beginGroup("mainWindow");
    settings.setValue("geometry", saveGeometry());
    settings.endGroup();

    settings.beginGroup("database");
    settings.setValue("currentDatabase", *currentDatabase);
    settings.endGroup();

    settingsWidget->saveSettingsSlot();
}

void MainWindow::readSettings()
{
    QSettings settings("giowisys", "Kontakter");

    settings.beginGroup("mainWindow");
    restoreGeometry(settings.value("geometry").toByteArray());
    settings.endGroup();

    settings.beginGroup("database");
    *currentDatabase = settings.value("currentDatabase").toString();
    settings.endGroup();
}

void MainWindow::checkFirstRun()
{
    firstRun = new bool;
    int dbCount = 0;
    QSettings settings("giowisys", "Kontakter");

    settings.beginGroup("database");
    dbCount = settings.value("dbCount").toInt();
    settings.endGroup();

    if (dbCount) {
        show();
        *firstRun = false;
        return;
    }

    *firstRun = true;
    newAccountSlot();
}

void MainWindow::checkDate()
{

    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup("kontakter");
    QDate date = settings.value("date", QDate()).toDate();

    QDate currentDate = QDate::currentDate();


    if (date.isNull()) {
        settings.setValue("date", currentDate);
        return;
    }

    if (settings.value("donateMessage", false).toBool())
        return;

    if (date.daysTo(currentDate) >= 14) {
        QDesktopServices::openUrl(QUrl("http://giowisys.com/products/kontakter/thanks/"));
        settings.setValue("donateMessage", true);
    }

    settings.endGroup();
}

void MainWindow::updateAccountsMenu()
{
    int dbCount;
    QStringList databases;
    QSettings settings("giowisys", "Kontakter");

    settings.beginGroup("database");
    dbCount = settings.value("dbCount").toInt();
    databases = settings.value("databases").toStringList();
    settings.endGroup();

    if (dbCount < 1) {
        disableAll(true);
    } else {
        disableAll(false);
    }

    int i = 0;
    int a = accountsActionList->size();
    if(a) {
        int j = 0;
        while (j < a) {
            delete accountsActionList->at(j);
            ++j;
        }
        accountsActionList->clear();
    }

    while (i < databases.size()) {
        QAction *tmp = new QAction(databases.at(i), accountActionGroup);
        tmp->setCheckable(true);
        connect(tmp, SIGNAL(triggered()), this, SLOT(accountChangeSlot()));
        accountsMenu->addAction(tmp);
        accountsActionList->append(tmp);
        if (databases.at(i) == *currentDatabase)
            tmp->setChecked(true);
        ++i;
    }
}

void MainWindow::checkUpdates()
{
    QSettings settings(tr("giowisys"), tr("Kontakter"));
    settings.beginGroup("kontakter");
    bool updates = settings.value("updates", true).toBool();
    settings.endGroup();

    if (updates)
        updateSlot();
}

void MainWindow::helloServer()
{
    if (!connectToServer())
        return;

    QByteArray outBlock;
    QDataStream out(&outBlock, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_5);
    out << qint64(0);
    out << *localServerPass << quint16(HELLO_REQUEST) << QByteArray("ping");
    out.device()->seek(0);
    out << qint64(outBlock.size() - sizeof(qint64));

    statusBar()->showMessage(tr("Sending HELLO request to server..."));

    tcpSocket->write(outBlock);

    if (!tcpSocket->waitForBytesWritten(10000)) {
        statusBar()->showMessage(tr("Server refused HELLO request!"));
        return;
    }

    if (!tcpSocket->waitForReadyRead(10000))
        return;

    QDataStream in(tcpSocket);
    in.setVersion(QDataStream::Qt_4_5);

    /* Kontakter packet format */
    qint64 size;
    QString serverPass;
    quint16 reqType;
    QByteArray data;

    in >> size;

    while (tcpSocket->bytesAvailable() < size) {
        if (!tcpSocket->waitForReadyRead()) {
            qDebug() << "Not enough bytes!";
            disconnectFromServer();
            return;
        }
    }

    in >> serverPass >> reqType >> data;

    if (reqType == HELLO_REQUEST) {

    } else if(reqType == ERROR_REPORT) {
        QMessageBox::critical(this, tr("Error"), QString(data));
    }

    disconnectFromServer();

    statusBar()->showMessage(tr("Server is online!"));
}


void MainWindow::goodbyeServer()
{
    if (!connectToServer())
        return;

    QByteArray outBlock;
    QDataStream out(&outBlock, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_5);
    out << qint64(0);
    out << *localServerPass << quint16(GOODBYE_REQUEST) << QByteArray("ping");
    out.device()->seek(0);
    out << qint64(outBlock.size() - sizeof(qint64));

    statusBar()->showMessage(tr("Sending GOODBYE request to server..."));

    tcpSocket->write(outBlock);

    if (!tcpSocket->waitForBytesWritten(10000)) {
        statusBar()->showMessage(tr("Server refused GOODBYE request!"));
    }

    disconnectFromServer();

    statusBar()->showMessage(tr("Conection closed!"));
}

/* _________________________________________________________________________ */
