/*
 *  Copyright (C) 2009  Giorgio Wicklein <g.wicklein@giowisys.com> 
 *  
 *  This file is part of Kontakter.
 *
 *  Kontakter is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Kontakter is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Kontakter.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QtGui>
#include <QtSql>

#include "newaccountwidget.h"
#include "pages/page_1.h"
#include "pages/localpage.h"
#include "pages/networkpage.h"
#include "mainwindow.h"

/* _________________________________PUBLIC__________________________________ */

//// constructor
NewAccountWidget::NewAccountWidget(QWidget *parent) : QWidget(parent)
{
//// stacked widget pages
    page_1 = new Page_1(this);
    localPage = new LocalPage(this);
    networkPage = new NetworkPage(this);

//// connections
    connect(page_1, SIGNAL(nextSignal()), this, SLOT(handlePage_1()));
    connect(page_1, SIGNAL(cancelSignal()), this, SIGNAL(cancelNewAccSlot()));
    connect(networkPage, SIGNAL(cancelSignal()), this, SIGNAL(cancelNewAccSlot()));
    connect(localPage, SIGNAL(createLocalAccountSignal()),
            this, SLOT(createLocalAccountSlot()));
    connect(networkPage, SIGNAL(createNetworkAccountSignal()),
            this, SLOT(createNetworkAccountSlot()));

//// stacked widget
    stackedWidget = new QStackedWidget(this);
    stackedWidget->addWidget(page_1);
    stackedWidget->addWidget(localPage);
    stackedWidget->addWidget(networkPage);

//// layout
    mainLayout = new QVBoxLayout;
    mainLayout->addWidget(stackedWidget);
    setLayout(mainLayout);

    setWindowTitle(tr("Create a new account"));
    setWindowModality(Qt::ApplicationModal);

//// set non resizable
    layout()->setSizeConstraint(QLayout::SetFixedSize);
}

/* _________________________________________________________________________ */
/* _________________________________EVENTS__________________________________ */

void NewAccountWidget::closeEvent(QCloseEvent *event)
{
    event->ignore();
}

/* _________________________________________________________________________ */
/* ______________________________PRIVATE SLOTS______________________________ */

//// this slot is activated by the next signal from page_1
//// according to the account type, it sets the correct page
//// 1 == localPage
//// 2 == networkPage
void NewAccountWidget::handlePage_1()
{
    if (page_1->localRadioButton->isChecked()) {
        stackedWidget->setCurrentIndex(1);
    } else {
        stackedWidget->setCurrentIndex(2);
    }
}

//// When localPage's finish button is clicked this
//// slot gets active
//// This slots creates a new local type account
void NewAccountWidget::createLocalAccountSlot()
{
    QSettings settings("giowisys", "Kontakter");
    int dbCount = 0;
    QStringList databases;

//// new account name
    QString accountName = page_1->accountLineEdit->text();

//// want to use password?
    bool usePass = page_1->passCheckBox->isChecked();

    settings.beginGroup("database");

//// how many databases/accounts are there?
    dbCount = settings.value("dbCount").toInt();

//// database/account names
    databases = settings.value("databases").toStringList();

//// we need to increment dbCount, cause we create new account
    ++dbCount;

//// add new account name to accounts list
    databases.append(accountName);

//// now write down updated settings
    settings.setValue("dbCount", dbCount);
    settings.setValue("databases", databases);
    settings.setValue("currentDatabase", accountName);

    settings.endGroup();

//// create account specific settings
    settings.beginGroup(accountName);
    settings.setValue("type", "local");
    settings.setValue("usePass", usePass);

    if (usePass) {
        QByteArray hash = QCryptographicHash::hash(
                QByteArray(page_1->passLineEdit->text().toStdString().c_str()),
                QCryptographicHash::Md5);
        settings.setValue("pass", hash);
    }
    settings.endGroup();

//// set up the new database
    createNewDatabase(accountName);

    emit completeSignal();
}

void NewAccountWidget::createNetworkAccountSlot()
{
    QSettings settings("giowisys", "Kontakter");
    int dbCount = 0;
    QStringList databases;

//// new account name
    QString accountName = page_1->accountLineEdit->text();

//// want to use password?
    bool usePass = page_1->passCheckBox->isChecked();

    settings.beginGroup("database");

//// how many databases/accounts are there?
    dbCount = settings.value("dbCount").toInt();

//// database/account names
    databases = settings.value("databases").toStringList();

//// we need to increment dbCount, cause we create new account
    ++dbCount;

//// add new account name to accounts list
    databases.append(accountName);

//// now write down updated settings
    settings.setValue("dbCount", dbCount);
    settings.setValue("databases", databases);
    settings.setValue("currentDatabase", accountName);

    settings.endGroup();

//// create account specific settings
    settings.beginGroup(accountName);
    settings.setValue("type", "network");
    settings.setValue("usePass", usePass);

    if (usePass) {
        QByteArray hash = QCryptographicHash::hash(
                QByteArray(page_1->passLineEdit->text().toStdString().c_str()),
                QCryptographicHash::Md5);
        settings.setValue("pass", hash);
    }

    settings.setValue("host", networkPage->hostLineEdit->text());
    settings.setValue("port", networkPage->portSpinBox->value());

    QByteArray serverHash = QCryptographicHash::hash(
                QByteArray(networkPage->passLineEdit->text().toStdString().c_str()),
                QCryptographicHash::Sha1);

    settings.setValue("serverPass", serverHash);
    settings.setValue("localRevision", 0);

    settings.endGroup();

//// set up the new database
    createNewDatabase(accountName);

    QMessageBox::information(this, tr("New account"),
                 tr("Account created successfully! Now you can use Kontakter, we hope you enjoy it!"));

    emit completeSignal();
}

/* _________________________________________________________________________ */
/* _________________________________PRIVATE_________________________________ */

//// this function is used to create a new sqlite database
//// and to create the necessary structure
void NewAccountWidget::createNewDatabase(QString name)
{
    QDir dir;

//// does the configuration directory already exists?
    bool kontakterDir = dir.exists(QDir::homePath() + "/.Kontakter/databases");

//// if not then create it
    if (!kontakterDir)
        dir.mkpath(QDir::homePath() + "/.Kontakter/databases");

    QString path = QDir::homePath() + "/.Kontakter/databases/";
    QSqlDatabase db(QSqlDatabase::addDatabase("QSQLITE"));
    db.setDatabaseName(path + name + ".sqlite");

    if (!db.open()) {
        QMessageBox::critical(this, tr("Database Error"),
                              db.lastError().text());
        qApp->quit();
    }

//// the brackets are necessary to close correctly the database
    {
        QSqlQuery query;

    //// create database structure
        query.exec("CREATE  TABLE \"main\".\"contacts\" (\"name\" VARCHAR, \"firstName\" VARCHAR, \"work\" VARCHAR, \"home\" VARCHAR, \"mobile\" VARCHAR, \"address\" VARCHAR, \"zip\" VARCHAR, \"town\" VARCHAR, \"email\" VARCHAR, \"web\" VARCHAR, \"notes\" VARCHAR, \"customCount\" INTEGER DEFAULT 0, \"customLabel1\" VARCHAR, \"customValue1\" VARCHAR, \"customLabel2\" VARCHAR, \"customValue2\" VARCHAR, \"customLabel3\" VARCHAR, \"customValue3\" VARCHAR, \"customLabel4\" VARCHAR, \"customValue4\" VARCHAR, \"customLabel5\" VARCHAR, \"customValue5\" VARCHAR, \"customLabel6\" VARCHAR, \"customValue6\" VARCHAR, \"id\" INTEGER PRIMARY KEY  AUTOINCREMENT , \"date\" DATETIME DEFAULT CURRENT_TIMESTAMP)");

    //// create index
        query.exec("CREATE  INDEX \"main\".\"idx_contacts\" ON \"contacts\" (\"name\" ASC, \"firstName\" ASC)");
    }

    db.close();
    db.removeDatabase("QSQLITE");
}

/* _________________________________________________________________________ */
