/*
 *  Copyright (C) 2009  Giorgio Wicklein <g.wicklein@giowisys.com> 
 *  
 *  This file is part of Kontakter.
 *
 *  Kontakter is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Kontakter is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Kontakter.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QtGui>

#include "settingswidget.h"
#include "mainwindow.h"

/* _________________________________PUBLIC__________________________________ */

//// constructor
SettingsWidget::SettingsWidget(QWidget *parent) : QWidget(parent)
{
//// main message
    mainLabel = new QLabel(tr("<h3>Kontakter settings</h3>"), this);
    mainLabel->setAlignment(Qt::AlignHCenter);

//// updates controls
    updatesLabel = new QLabel(tr("Check for updates: "), this);
    updatesComboBox = new QComboBox(this);
    updatesComboBox->addItem(tr("at startup"));
    updatesComboBox->addItem(tr("manually"));

//// blur background (translucent)
    blurBackgroundLabel = new QLabel(tr("Translucent Background (Windows only): "), this);
    blurBackgroundBox = new QComboBox(this);
    blurBackgroundBox->addItem(tr("Enabled"));
    blurBackgroundBox->addItem(tr("Disabled"));

#ifndef Q_WS_WIN
    blurBackgroundBox->setEnabled(false);
#endif

//// password controls
    usePassBox = new QCheckBox(tr("Enable password for this account: "), this);
    passLineEdit = new QLineEdit(this);
    passLineEdit->setEchoMode(QLineEdit::Password);

    networkGroupBox = new QGroupBox(tr("Server settings"), this);

    hostLabel = new QLabel(tr("Host address: "), this);
    hostLineEdit = new QLineEdit(this);

    portLabel = new QLabel(tr("Port: "), this);

    portSpinBox = new QSpinBox(this);
    portSpinBox->setRange(60, 65535);
    portSpinBox->setValue(1337);

    serverPassLabel = new QLabel(tr("Server password: "), this);

    serverPassLineEdit = new QLineEdit(this);
    serverPassLineEdit->setEchoMode(QLineEdit::Password);

//// back button
    backButton = new QPushButton(tr("&Back"), this);

//// pointer to access MainWindow's stuff
    mW = qobject_cast<MainWindow*>(parentWidget());

//// connections
    connect(backButton, SIGNAL(clicked()), mW, SLOT(settingsActionSlot()));
    connect(usePassBox, SIGNAL(toggled(bool)), passLineEdit, SLOT(setEnabled(bool)));
    connect(usePassBox, SIGNAL(toggled(bool)), passLineEdit, SLOT(clear()));
    connect(serverPassLineEdit, SIGNAL(editingFinished()), this, SLOT(restartMessageSlot()));
    connect(hostLineEdit, SIGNAL(editingFinished()), this, SLOT(restartMessageSlot()));

//// layouts
    backLayout = new QHBoxLayout;
    backLayout->addWidget(backButton);
    backLayout->addStretch();

    settingsLayout = new QGridLayout;
    settingsLayout->addWidget(updatesLabel, 0, 0);
    settingsLayout->addWidget(updatesComboBox, 0, 1);
    settingsLayout->addWidget(blurBackgroundLabel, 1, 0);
    settingsLayout->addWidget(blurBackgroundBox, 1, 1);
    settingsLayout->addWidget(usePassBox, 2, 0);
    settingsLayout->addWidget(passLineEdit, 2, 1);

    networkLayout = new QGridLayout;
    networkLayout->addWidget(hostLabel, 0, 0);
    networkLayout->addWidget(hostLineEdit, 0, 1);
    networkLayout->addWidget(portLabel, 1, 0);
    networkLayout->addWidget(portSpinBox, 1, 1);
    networkLayout->addWidget(serverPassLabel, 2, 0);
    networkLayout->addWidget(serverPassLineEdit, 2, 1);

    networkGroupBox->setLayout(networkLayout);

    mainLayout = new QVBoxLayout;
    mainLayout->addWidget(mainLabel);
    mainLayout->addLayout(settingsLayout);
    mainLayout->addWidget(networkGroupBox);
    mainLayout->addStretch();
    mainLayout->addLayout(backLayout);

    setLayout(mainLayout);

//// load saved value to set up all controls
    loadSettings();
}

//// assings all controls the saved values
void SettingsWidget::loadSettings()
{
    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup("kontakter");
    bool updates = settings.value("updates", true).toBool();
    bool blurBackground = settings.value("blurBackground", true).toBool();
    settings.endGroup();

    if (updates)
        updatesComboBox->setCurrentIndex(0);
    else
        updatesComboBox->setCurrentIndex(1);

    if (blurBackground)
        blurBackgroundBox->setCurrentIndex(0);
    else
        blurBackgroundBox->setCurrentIndex(1);

    settings.beginGroup(*mW->currentDatabase);

    bool usePass = settings.value("usePass").toBool();
    if (usePass) {
        usePassBox->setChecked(true);
        passLineEdit->setText("password");
        passLineEdit->setEnabled(true);
    } else {
        usePassBox->setChecked(false);
        passLineEdit->clear();
        passLineEdit->setEnabled(false);
    }

    if (settings.value("type").toString() == QString("network")) {
        networkGroupBox->setEnabled(true);
        hostLineEdit->setText(settings.value("host").toString());
        portSpinBox->setValue(settings.value("port").toInt());
        serverPassLineEdit->setText("password");
    } else {
        networkGroupBox->setEnabled(false);
        hostLineEdit->clear();
        portSpinBox->setValue(1337);
        serverPassLineEdit->clear();
    }

    settings.endGroup();
}

/* _________________________________________________________________________ */
/* ______________________________PRIVATE SLOTS______________________________ */

//// reads current values from all controls and saves them
void SettingsWidget::saveSettingsSlot()
{
    QSettings settings("giowisys", "Kontakter");
    settings.beginGroup("kontakter");

//// current index is true when manually is selected (index 1)
    if (updatesComboBox->currentIndex())
        settings.setValue("updates", false);
    else
        settings.setValue("updates", true);

    if (blurBackgroundBox->currentIndex())
        settings.setValue("blurBackground", false);
    else
        settings.setValue("blurBackground", true);

    settings.endGroup();
    settings.beginGroup(*mW->currentDatabase);

    if (usePassBox->isChecked()) {
        if (passLineEdit->text().isEmpty()) {
            settings.setValue("usePass", false);
        }
        else {
            settings.setValue("usePass", true);
            QString pass = passLineEdit->text();
            if (!(pass == "password")) {
                QByteArray hash = QCryptographicHash::hash(
                        QByteArray(pass.toStdString().c_str()),
                        QCryptographicHash::Md5);
                settings.setValue("pass", hash);
            }
        }
    } else {
        settings.setValue("usePass", false);
    }

    if (settings.value("type").toString() == QString("network")) {
        settings.setValue("host", hostLineEdit->text());
        settings.setValue("port", portSpinBox->value());

        if (serverPassLineEdit->text() != "password") {
            QByteArray serverHash = QCryptographicHash::hash(
                QByteArray(serverPassLineEdit->text().toStdString().c_str()),
                QCryptographicHash::Sha1);

            settings.setValue("serverPass", serverHash);
        }
    }

    settings.endGroup();
}

void SettingsWidget::restartMessageSlot()
{
    QMessageBox::information(this, tr("Restart required"),
                             tr("Please restart Kontakter to apply new settings"));
}

/* _________________________________________________________________________ */
/* ________________________________PRIVATE__________________________________ */



/* _________________________________________________________________________ */
