/*
 *  Copyright (C) 2009  Giorgio Wicklein <g.wicklein@giowisys.com>
 *
 *  This file is part of Kontakter.
 *
 *  Kontakter is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Kontakter is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Kontakter.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QtGui>
#include <QtSql>

#include "mainwindow.h"
#include "serverwidget.h"
#include "server.h"

/* _________________________________PUBLIC__________________________________ */

MainWindow::MainWindow()
{
    serverWidget = new ServerWidget(this);

    checkFirstRun();
    createActions();
    createMenus();
    createConnections();
    createStatusBar();
    createTrayIcon();
    readSettings();

    setCentralWidget(serverWidget);
    setWindowTitle(tr("Kontakter-server"));
}

/* _________________________________________________________________________ */
/* _________________________________EVENTS__________________________________ */

void MainWindow::closeEvent(QCloseEvent *event)
{
    if (isVisible()) {
        hide();
        trayIcon->showMessage(tr("Kontakter-server"), tr("Running..."),
                              QSystemTrayIcon::NoIcon);
        event->ignore();
    }
    else {
        writeSettings();
        event->accept();
    }
}

/* _________________________________________________________________________ */
/* ______________________________PRIVATE SLOTS______________________________ */

void MainWindow::quitSlot()
{
    if (isClientConnected()) {
        int r = QMessageBox::warning(this, tr("Connected clients"),
                                      tr("There are still connected clients, are you sure you want to close the server?\n"
                                         "This could provocate data loss and an unaspected behaviour"),
                                      QMessageBox::No | QMessageBox::Yes);

        if (r == QMessageBox::No)
            return;
    }

    writeSettings();
    qApp->quit();
}

void MainWindow::startSlot()
{
    if (serverWidget->passLineEdit->text().isEmpty()) {
        statusBar()->showMessage(tr("Error: password cannot be empty!"));
        return;
    }

    writeSettings();

    serverWidget->startButton->setVisible(false);
    serverWidget->stopButton->setVisible(true);
    serverWidget->portSpinBox->setEnabled(false);
    serverWidget->passLineEdit->setEnabled(false);

    kontakterServer = new Server(this);
    if (!kontakterServer->listen(QHostAddress::Any, serverWidget->portSpinBox->value())) {
        QMessageBox::critical(this, "Server listen failed",
                              tr("Server failed to start listening, maybe server already running or port already in use?"));
        stopSlot();
        return;
    }

    statusBar()->showMessage(tr("Server started: listening!"));

    resetDbAction->setEnabled(false);
    exportDbAction->setEnabled(false);
    importDbAction->setEnabled(false);
}

void MainWindow::stopSlot()
{
    if (isClientConnected()) {
        int r = QMessageBox::warning(this, tr("Connected clients"),
                                      tr("There are still connected clients, are you sure you want to close the server?\n"
                                         "This could provocate data loss and an unaspected behaviour"),
                                      QMessageBox::No | QMessageBox::Yes);

        if (r == QMessageBox::No)
            return;
    }

    serverWidget->usersLabel->setText("0");
    serverWidget->startButton->setVisible(true);
    serverWidget->stopButton->setVisible(false);
    serverWidget->portSpinBox->setEnabled(true);
    serverWidget->passLineEdit->setEnabled(true);

    kontakterServer->close();
    delete kontakterServer;

    statusBar()->showMessage(tr("Server stopped: not listening!"));

    resetDbAction->setEnabled(true);
    exportDbAction->setEnabled(true);
    importDbAction->setEnabled(true);
}

void MainWindow::documentationSlot()
{
    QDesktopServices::openUrl(QUrl("http://giowisys.com/products/kontakter/doc/"));
}

void MainWindow::trayActivated(QSystemTrayIcon::ActivationReason reason)
{
    switch (reason) {
        case QSystemTrayIcon::DoubleClick:
            if (isHidden())
                show();
            else
                hide();
            break;
        default:
            ;
     }
}

void MainWindow::resetDatabaseSlot()
{
    int r = QMessageBox::warning(this, tr("Confirm Database Reset"),
                                 tr("Are you sure you want to delete all contacts from the database?"),
                                 QMessageBox::No | QMessageBox::Yes);

    if (r == QMessageBox::No)
        return;

    QString dbName = QDir::homePath() + "/.Kontakter/server/server.sqlite";

    QDir dir;
    if (!dir.remove(dbName)) {
        QMessageBox::critical(this, tr("File error"), tr("Cannot delete the database!"));
    } else {
        QMessageBox::information(this, tr("Database reset"), tr("Please restart the application!"));
    }

    QSettings settings("giowisys", "Kontakter-server");
    settings.beginGroup("server");
    settings.setValue("serverRevision", 0);
    settings.endGroup();
}

void MainWindow::importDbSlot()
{
    QString inFile = QFileDialog::getOpenFileName(this, tr("Import database file"),
                                             QDir::homePath(),
                                             tr("SQLITE files (*.sqlite)"));

    if (inFile.isEmpty())
        return;

    bool r = QMessageBox::warning(this, tr("Import database"),
                                  tr("Are you sure you want to substitute the "
                                     "current database with the external one?"),
                                  QMessageBox::No | QMessageBox::Yes);

    if (r == QMessageBox::No)
        return;

    QString fullPath = QDir::homePath() + "/.Kontakter/server/server.sqlite";
    QDir dir;

    if (!dir.remove(fullPath)) {
        QMessageBox::critical(this, tr("File error"),
                              tr("Cannot delete the database!"));
        return;
    }

    if (!QFile::copy(inFile, fullPath)) {
        QMessageBox::critical(this, tr("File error"),
                              tr("Cannot write the database!"));
        return;
    }

    QSettings settings("giowisys", "Kontakter-server");
    settings.beginGroup("server");
    quint64 serverRevision = settings.value("serverRevision", 0).toULongLong();
    ++serverRevision;
    settings.setValue("serverRevision", serverRevision);
    settings.endGroup();

    statusBar()->showMessage(tr("Database imported successfully!"));
}

void MainWindow::exportDbSlot()
{
    QString outFile =
                QFileDialog::getSaveFileName(this, tr("Export database file"),
                                             QDir::homePath(),
                                             tr("SQLITE files (*.sqlite)"));
    if (outFile.isEmpty())
            return;

    if (!outFile.contains(".sqlite"))
            outFile.append(".sqlite");

    QFile fileDb(QDir::homePath() + "/.Kontakter/server/server.sqlite");

    bool cp = fileDb.copy(outFile);

    if (!cp) {
        QMessageBox::critical(this, tr("Error in database export"),
                              tr("An error occurred while exporting the database!"));
        return;
    }
    statusBar()->showMessage(tr("Database exported successfully!"));
}

/* _________________________________________________________________________ */
/* _________________________________PRIVATE_________________________________ */

void MainWindow::createActions()
{
    importDbAction = new QAction(tr("Import database"), this);
    importDbAction->setStatusTip(tr("Import an external database, including all contacts"));

    exportDbAction = new QAction(tr("Export database"), this);
    exportDbAction->setStatusTip(tr("Export current database, including all contacts"));

    exitAction = new QAction(tr("&Exit"), this);
    exitAction->setShortcut(tr("CTRL+Q"));

    resetDbAction = new QAction(tr("Database reset"), this);
    resetDbAction->setStatusTip(tr("Resets the database including all contacts"));

    documentationAction = new QAction(tr("&Online documentation"), this);
    documentationAction->setShortcut(QKeySequence::HelpContents);
}

void MainWindow::createMenus()
{
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(importDbAction);
    fileMenu->addAction(exportDbAction);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAction);

    editMenu = menuBar()->addMenu(tr("&Edit"));
    editMenu->addAction(resetDbAction);

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(documentationAction);
}

void MainWindow::createConnections()
{
    connect(serverWidget->quitButton, SIGNAL(clicked()), this, SLOT(quitSlot()));
    connect(serverWidget->startButton, SIGNAL(clicked()), this, SLOT(startSlot()));
    connect(serverWidget->stopButton, SIGNAL(clicked()), this, SLOT(stopSlot()));
    connect(exitAction, SIGNAL(triggered()), this, SLOT(quitSlot()));
    connect(resetDbAction, SIGNAL(triggered()), this, SLOT(resetDatabaseSlot()));
    connect(documentationAction, SIGNAL(triggered()), this, SLOT(documentationSlot()));
    connect(importDbAction, SIGNAL(triggered()), this, SLOT(importDbSlot()));
    connect(exportDbAction, SIGNAL(triggered()), this, SLOT(exportDbSlot()));
}

void MainWindow::createStatusBar()
{
    statusBar()->showMessage(tr("Server not started!"));
}

void MainWindow::createTrayIcon()
{
    showAction = new QAction(tr("Show Kontakter-server"), this);
    connect(showAction, SIGNAL(triggered()), this, SLOT(show()));
    hideAction = new QAction(tr("Hide Kontakter-server"), this);
    connect(hideAction, SIGNAL(triggered()), this, SLOT(hide()));

    trayMenu = new QMenu(this);
    trayMenu->addAction(showAction);
    trayMenu->addAction(hideAction);
    trayMenu->addSeparator();
    trayMenu->addAction(exitAction);

    trayIcon = new QSystemTrayIcon(this);
    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
             this, SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));
    trayIcon->setContextMenu(trayMenu);
    trayIcon->setIcon(QIcon(":/icons/server.png"));
    trayIcon->setToolTip(tr("Kontakter-server"));
    trayIcon->show();
}

void MainWindow::writeSettings()
{
    QSettings settings("giowisys", "Kontakter-server");

    settings.beginGroup("mainWindow");
    settings.setValue("geometry", saveGeometry());
    settings.endGroup();

    settings.beginGroup("server");

    QString pass = serverWidget->passLineEdit->text();

    if ((pass != "password") && (!pass.isEmpty())) {
        QByteArray hash = QCryptographicHash::hash(
                        QByteArray(pass.toStdString().c_str()),
                        QCryptographicHash::Sha1);
        settings.setValue("password", hash);
    }

    settings.setValue("port", serverWidget->portSpinBox->value());

    settings.setValue("start", serverWidget->startCheckBox->isChecked());

    settings.endGroup();
}

void MainWindow::readSettings()
{
    QSettings settings("giowisys", "Kontakter-server");

    settings.beginGroup("mainWindow");
    restoreGeometry(settings.value("geometry").toByteArray());
    settings.endGroup();

    settings.beginGroup("server");
    QByteArray pass = settings.value("password").toByteArray();

    if (pass.isEmpty())
        serverWidget->passLineEdit->setText("");
    else
        serverWidget->passLineEdit->setText("password");

    int port = settings.value("port", 1337).toInt();
    serverWidget->portSpinBox->setValue(port);

    if (settings.value("start", false).toBool()) {
        serverWidget->startCheckBox->setChecked(true);
        startSlot();
    }

    settings.endGroup();
}

bool MainWindow::isClientConnected()
{
    int clients = serverWidget->usersLabel->text().toInt();

    if (clients)
        return true;
    else
        return false;
}

void MainWindow::checkFirstRun()
{
    QDir dir;

    bool serverDir = dir.exists(QDir::homePath() + "/.Kontakter/server");

    if (!serverDir) {
        dir.mkpath(QDir::homePath() + "/.Kontakter/server");

        QSettings settings("giowisys", "Kontakter-server");
        settings.beginGroup("server");
        settings.setValue("serverRevision", 0);
        settings.endGroup();
    }

    QString path = QDir::homePath() + "/.Kontakter/server/";
    QSqlDatabase db(QSqlDatabase::addDatabase("QSQLITE"));
    db.setDatabaseName(path + "server.sqlite");

    if (!db.open()) {
        QMessageBox::critical(this, tr("Database Error"),
                              db.lastError().text());
        return;
    }

//// the brackets are necessary to close correctly the database
    {
        QSqlQuery query;

    //// create database structure
        query.exec("CREATE  TABLE \"main\".\"contacts\" (\"name\" VARCHAR, \"firstName\" VARCHAR, \"work\" VARCHAR, \"home\" VARCHAR, \"mobile\" VARCHAR, \"address\" VARCHAR, \"zip\" VARCHAR, \"town\" VARCHAR, \"email\" VARCHAR, \"web\" VARCHAR, \"notes\" VARCHAR, \"customCount\" INTEGER DEFAULT 0, \"customLabel1\" VARCHAR, \"customValue1\" VARCHAR, \"customLabel2\" VARCHAR, \"customValue2\" VARCHAR, \"customLabel3\" VARCHAR, \"customValue3\" VARCHAR, \"customLabel4\" VARCHAR, \"customValue4\" VARCHAR, \"customLabel5\" VARCHAR, \"customValue5\" VARCHAR, \"customLabel6\" VARCHAR, \"customValue6\" VARCHAR, \"id\" INTEGER PRIMARY KEY  AUTOINCREMENT , \"date\" DATETIME DEFAULT CURRENT_TIMESTAMP)");

    //// create index
        query.exec("CREATE  INDEX \"main\".\"idx_contacts\" ON \"contacts\" (\"name\" ASC, \"firstName\" ASC)");
    }

    db.close();
    db.removeDatabase("QSQLITE");
}

/* _________________________________________________________________________ */
