/*
 *  Copyright (C) 2009  Giorgio Wicklein <g.wicklein@giowisys.com>
 *
 *  This file is part of Kontakter.
 *
 *  Kontakter is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Kontakter is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Kontakter.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QtNetwork>
#include <QtGui>

#include "serverthread.h"
#include "definitions.h"

ServerThread::ServerThread(int socketDescriptor, QObject *parent)
        : QThread(parent), socketDescriptor(socketDescriptor)
{
}

void ServerThread::run()
{
    QTcpSocket tcpSocket;

    if (!tcpSocket.setSocketDescriptor(socketDescriptor)) {
        emit error(tcpSocket.errorString());
        return;
    }

    if (!tcpSocket.waitForReadyRead()) {
        tcpSocket.disconnectFromHost();
        tcpSocket.waitForDisconnected();
        return;
    }

    QDataStream in(&tcpSocket);
    in.setVersion(QDataStream::Qt_4_5);

    /* Kontakter packet format */
    qint64 size;
    QString pass;
    quint16 reqType;
    QByteArray data;

    in >> size;

    while (tcpSocket.bytesAvailable() < size) {
        if (!tcpSocket.waitForReadyRead()) {
            qDebug() << "Not enough bytes!";
            tcpSocket.disconnectFromHost();
            tcpSocket.waitForDisconnected();
            return;
        }
    }

    in >> pass >> reqType >> data;

    QSettings settings("giowisys", "Kontakter-server");
    settings.beginGroup("server");
    QString serverPass = settings.value("password").toString();
    settings.endGroup();

    if (pass != serverPass) {

        QByteArray outBlock;
        QDataStream out(&outBlock, QIODevice::WriteOnly);
        out.setVersion(QDataStream::Qt_4_5);
        out << qint64(0);
        out << serverPass << quint16(ERROR_REPORT) << QByteArray(QString(tr("Wrong password!")).toStdString().c_str());
        out.device()->seek(0);
        out << qint64(outBlock.size() - sizeof(qint64));

        tcpSocket.write(outBlock);

        tcpSocket.waitForBytesWritten();
        tcpSocket.disconnectFromHost();
        tcpSocket.waitForDisconnected();

        return;
    }

    if (reqType == HELLO_REQUEST) {

        emit newClientSignal();

        QByteArray outBlock;
        QDataStream out(&outBlock, QIODevice::WriteOnly);
        out.setVersion(QDataStream::Qt_4_5);
        out << qint64(0);
        out << serverPass << quint16(HELLO_REQUEST) << QByteArray("pong");
        out.device()->seek(0);
        out << qint64(outBlock.size() - sizeof(qint64));

        tcpSocket.write(outBlock);

    } else if (reqType == GOODBYE_REQUEST) {

        emit removeClientSignal();

    } else if (reqType == REVISION_REQUEST) {

        QSettings setting("giowisys", "Kontakter-server");
        settings.beginGroup("server");
        quint64 serverRevision = settings.value("serverRevision", 0).toULongLong();
        settings.endGroup();

        QByteArray outBlock;
        QDataStream out(&outBlock, QIODevice::WriteOnly);
        out.setVersion(QDataStream::Qt_4_5);
        out << qint64(0);
        out << serverPass << quint16(REVISION_REQUEST) << QByteArray(QString::number(serverRevision).toStdString().c_str());
        out.device()->seek(0);
        out << qint64(outBlock.size() - sizeof(qint64));

        tcpSocket.write(outBlock);

    } else if (reqType == ADD_REQUEST) {

        emit addContactSignal(data);

        QSettings setting("giowisys", "Kontakter-server");
        settings.beginGroup("server");
        quint64 serverRevision = settings.value("serverRevision", 0).toULongLong();
        ++serverRevision;
        settings.setValue("serverRevision", serverRevision);
        settings.endGroup();

        QByteArray outBlock;
        QDataStream out(&outBlock, QIODevice::WriteOnly);
        out.setVersion(QDataStream::Qt_4_5);
        out << qint64(0);
        out << serverPass << quint16(ADD_REQUEST) << QByteArray();
        out.device()->seek(0);
        out << qint64(outBlock.size() - sizeof(qint64));

        tcpSocket.write(outBlock);

    } else if (reqType == DEL_REQUEST) {

        emit delContactSignal(data);

        QSettings setting("giowisys", "Kontakter-server");
        settings.beginGroup("server");
        quint64 serverRevision = settings.value("serverRevision", 0).toULongLong();
        ++serverRevision;
        settings.setValue("serverRevision", serverRevision);
        settings.endGroup();

        QByteArray outBlock;
        QDataStream out(&outBlock, QIODevice::WriteOnly);
        out.setVersion(QDataStream::Qt_4_5);
        out << qint64(0);
        out << serverPass << quint16(DEL_REQUEST) << QByteArray();
        out.device()->seek(0);
        out << qint64(outBlock.size() - sizeof(qint64));

        tcpSocket.write(outBlock);

    } else if (reqType == EDIT_REQUEST) {

        emit editContactSignal(data);

        QSettings setting("giowisys", "Kontakter-server");
        settings.beginGroup("server");
        quint64 serverRevision = settings.value("serverRevision", 0).toULongLong();
        ++serverRevision;
        settings.setValue("serverRevision", serverRevision);
        settings.endGroup();

        QByteArray outBlock;
        QDataStream out(&outBlock, QIODevice::WriteOnly);
        out.setVersion(QDataStream::Qt_4_5);
        out << qint64(0);
        out << serverPass << quint16(EDIT_REQUEST) << QByteArray();
        out.device()->seek(0);
        out << qint64(outBlock.size() - sizeof(qint64));

        tcpSocket.write(outBlock);

    } else if (reqType == GET_DB) {

        QFile inDb(QDir::homePath() + "/.Kontakter/server/server.sqlite");

        if (!inDb.open(QIODevice::ReadOnly)) {
            QMessageBox::critical(0, tr("File error"), tr("Cannot open the database to send it!"));
            tcpSocket.disconnectFromHost();
            inDb.close();
            return;
        }

        QByteArray database = inDb.readAll();

        qDebug() << database.size();

        if (database.size() < 1) {
            QMessageBox::critical(0, tr("File error"), tr("Cannot send the database!"));
            tcpSocket.disconnectFromHost();
            inDb.close();
            return;
        }

        QByteArray outBlock;
        QDataStream out(&outBlock, QIODevice::WriteOnly);
        out.setVersion(QDataStream::Qt_4_5);
        out << qint64(0);
        out << serverPass << quint16(GET_DB) << database;
        out.device()->seek(0);
        out << qint64(outBlock.size() - sizeof(qint64));

        tcpSocket.write(outBlock);
        inDb.close();

    } else {
        qDebug() << "Unknown request type!";
    }

    tcpSocket.waitForBytesWritten();   
    tcpSocket.disconnectFromHost();
    tcpSocket.waitForDisconnected();
}
